# Copyright 2021-2022 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import json
from unittest.mock import Mock, patch

import anyio
import pytest
from tenacity import RetryError

from cerbos.sdk.client import *
from cerbos.sdk.model import *

pytestmark = pytest.mark.anyio


class TestCerbosClient:
    def test_is_healthy(self, cerbos_client: CerbosClient):
        have = cerbos_client.is_healthy()
        assert have == True

    def test_is_allowed(
        self,
        cerbos_client: CerbosClient,
        principal_john: Principal,
        resource_john_leave_req: Resource,
    ):
        have = cerbos_client.is_allowed(
            "view:public", principal_john, resource_john_leave_req
        )
        assert have == True

    def test_principal_context_is_allowed(
        self, principal_ctx: PrincipalContext, resource_john_leave_req: Resource
    ):
        have = principal_ctx.is_allowed("view:public", resource_john_leave_req)
        assert have == True

    def test_check_resources(
        self,
        cerbos_client: CerbosClient,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        have = cerbos_client.check_resources(principal_john, resource_list)
        _assert_check_resources(have)

    def test_check_resources_empty_resources(
        self, cerbos_client: CerbosClient, principal_john: Principal
    ):
        have = cerbos_client.check_resources(principal_john, ResourceList())
        _assert_check_resources_empty_resources(have)

    def test_plan_resources(
        self,
        cerbos_client: CerbosClient,
        principal_maggie: Principal,
        resourcedesc_leave_req: ResourceDesc,
    ):
        have = cerbos_client.plan_resources(
            "approve", principal_maggie, resourcedesc_leave_req
        )
        _assert_plan_resources(have)

    def test_plan_resources_validation(
        self,
        cerbos_client: CerbosClient,
        principal_maggie_invalid: Principal,
        resourcedesc_leave_req_invalid: ResourceDesc,
    ):
        have = cerbos_client.plan_resources(
            "approve", principal_maggie_invalid, resourcedesc_leave_req_invalid
        )
        _assert_plan_resources_validation(have)

    def test_check_resources_with_output(
        self,
        cerbos_client: CerbosClient,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        have = cerbos_client.check_resources(principal_john, resource_list)
        _assert_check_resources_with_output(have)

    @patch("httpx.Client.post")
    def test_request_retries_on_failure(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 200

        post_mock.side_effect = [
            IOError(),
            httpx.Response(
                status_code=status_code,
                text=json.dumps(
                    {
                        "request_id": "123",
                        "code": status_code,
                        "message": "success",
                    }
                ),
            ),
        ]

        client = CerbosClient("http://localhost:3592", request_retries=1, debug=True)

        resp = client.check_resources(principal_john, resource_list)

        assert resp.status_code == status_code
        assert post_mock.call_count == 2

    @patch("httpx.Client.post")
    def test_request_retries_not_if_zero(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 400

        post_mock.side_effect = [
            IOError(),
            httpx.Response(
                status_code=status_code,
                text=json.dumps(
                    {
                        "request_id": "123",
                        "code": status_code,
                        "message": "success",
                    }
                ),
            ),
        ]

        client = CerbosClient("http://localhost:3592", request_retries=0, debug=True)

        with pytest.raises(IOError):
            resp = client.check_resources(principal_john, resource_list)

            assert resp.failed()
            assert resp.status_code == status_code
            assert post_mock.call_count == 1

    @patch("httpx.Client.post")
    def test_request_retries_not_on_success(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 200

        post_mock.return_value = httpx.Response(
            status_code=status_code,
            text=json.dumps(
                {
                    "request_id": "123",
                    "code": status_code,
                    "message": "success",
                }
            ),
        )

        client = CerbosClient("http://localhost:3592", request_retries=1, debug=True)

        resp = client.check_resources(principal_john, resource_list)

        assert resp.status_code == status_code
        assert post_mock.call_count == 1

    @patch("socket.create_connection")
    def test_connection_retries(
        self,
        socket_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        n_retries = 3
        n_requests = n_retries + 1

        socket_mock.side_effect = [ConnectionRefusedError()] * n_requests

        client = CerbosClient(
            "http://localhost:3592", connection_retries=n_retries, debug=True
        )

        with pytest.raises(httpx.ConnectError):
            client.check_resources(principal_john, resource_list)

        assert socket_mock.call_count == n_requests


class TestPrincipalContext:
    def test_is_allowed(
        self, principal_ctx: PrincipalContext, resource_john_leave_req: Resource
    ):
        have = principal_ctx.is_allowed("view:public", resource_john_leave_req)
        assert have == True

    def test_check_resources(
        self, principal_ctx: PrincipalContext, resource_list: ResourceList
    ):
        have = principal_ctx.check_resources(resource_list)
        _assert_check_resources(have)

    def test_check_resources_with_output(
        self,
        cerbos_client: CerbosClient,
        principal_donald: Principal,
        resource_list: ResourceList,
    ):
        principal_ctx_override = cerbos_client.with_principal(principal_donald)
        have = principal_ctx_override.check_resources(resource_list)
        _assert_check_resources_principal_override_with_output(have)

    def test_plan_resources(
        self, principal_ctx: PrincipalContext, resourcedesc_leave_req: ResourceDesc
    ):
        have = principal_ctx.plan_resources("view:*", resourcedesc_leave_req)
        _assert_plan_resources(have)


class TestAsyncCerbosClient:
    async def test_is_healthy(self, cerbos_async_client: AsyncCerbosClient):
        have = await cerbos_async_client.is_healthy()
        assert have == True

    async def test_is_allowed(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_john: Principal,
        resource_john_leave_req: Resource,
    ):
        have = await cerbos_async_client.is_allowed(
            "view:public", principal_john, resource_john_leave_req
        )
        assert have == True

    async def test_principal_context_is_allowed(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resource_john_leave_req: Resource,
    ):
        have = await async_principal_ctx.is_allowed(
            "view:public", resource_john_leave_req
        )
        assert have == True

    async def test_check_resources(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        have = await cerbos_async_client.check_resources(principal_john, resource_list)
        _assert_check_resources(have)

    async def test_check_resources_empty_resources(
        self, cerbos_async_client: AsyncCerbosClient, principal_john: Principal
    ):
        have = await cerbos_async_client.check_resources(principal_john, ResourceList())
        _assert_check_resources_empty_resources(have)

    async def test_plan_resources(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_maggie: Principal,
        resourcedesc_leave_req: ResourceDesc,
    ):
        have = await cerbos_async_client.plan_resources(
            "approve", principal_maggie, resourcedesc_leave_req
        )
        _assert_plan_resources(have)

    async def test_plan_resources_validation(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_maggie_invalid: Principal,
        resourcedesc_leave_req_invalid: ResourceDesc,
    ):
        have = await cerbos_async_client.plan_resources(
            "approve", principal_maggie_invalid, resourcedesc_leave_req_invalid
        )
        _assert_plan_resources_validation(have)

    async def test_check_resources_with_output(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        have = await cerbos_async_client.check_resources(principal_john, resource_list)
        _assert_check_resources_with_output(have)

    @patch("httpx.AsyncClient.post")
    async def test_request_retries_on_failure(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 200
        post_mock.side_effect = [
            IOError(),
            httpx.Response(
                status_code=status_code,
                text=json.dumps(
                    {
                        "request_id": "123",
                        "code": status_code,
                        "message": "success",
                    }
                ),
            ),
        ]

        client = AsyncCerbosClient(
            "http://localhost:3592", request_retries=1, debug=True
        )

        # TODO SamL 2023-01-03: My local machine is raising this error during this test, but the GH test runners are not. Needs to be investigated.
        # with pytest.raises(IOError):
        resp = await client.check_resources(principal_john, resource_list)

        assert resp.status_code == status_code
        assert post_mock.call_count == 2

    @patch("httpx.AsyncClient.post")
    async def test_request_retries_not_if_zero(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 400

        post_mock.side_effect = [
            IOError(),
            httpx.Response(
                status_code=status_code,
                text=json.dumps(
                    {
                        "request_id": "123",
                        "code": status_code,
                        "message": "success",
                    }
                ),
            ),
        ]

        client = AsyncCerbosClient(
            "http://localhost:3592", request_retries=0, debug=True
        )

        with pytest.raises(IOError):
            resp = await client.check_resources(principal_john, resource_list)

            assert resp.failed()
            assert resp.status_code == status_code
            assert post_mock.call_count == 1

    @patch("httpx.AsyncClient.post")
    async def test_request_retries_not_on_success(
        self,
        post_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        status_code = 200

        post_mock.return_value = httpx.Response(
            status_code=status_code,
            text=json.dumps(
                {
                    "request_id": "123",
                    "code": status_code,
                    "message": "success",
                }
            ),
        )

        client = AsyncCerbosClient(
            "http://localhost:3592", request_retries=1, debug=True
        )

        resp = await client.check_resources(principal_john, resource_list)

        assert resp.status_code == status_code
        assert post_mock.call_count == 1

    @patch("anyio._backends._asyncio.connect_tcp")
    async def test_connection_retries(
        self,
        tcp_mock,
        principal_john: Principal,
        resource_list: ResourceList,
    ):
        n_retries = 3
        n_requests = n_retries + 1

        tcp_mock.side_effect = [ConnectionRefusedError()] * n_requests

        client = AsyncCerbosClient(
            "http://127.0.0.1:3592", connection_retries=n_retries, debug=True
        )

        with pytest.raises(httpx.ConnectError):
            await client.check_resources(principal_john, resource_list)

        assert tcp_mock.call_count == n_requests


class TestAsyncAsyncPrincipalContext:
    async def test_is_allowed(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resource_john_leave_req: Resource,
    ):
        have = await async_principal_ctx.is_allowed(
            "view:public", resource_john_leave_req
        )
        assert have == True

    async def test_check_resources(
        self, async_principal_ctx: AsyncPrincipalContext, resource_list: ResourceList
    ):
        have = await async_principal_ctx.check_resources(resource_list)
        _assert_check_resources(have)

    async def test_check_resources_with_output(
        self,
        cerbos_async_client: AsyncCerbosClient,
        principal_donald: Principal,
        resource_list: ResourceList,
    ):
        async_principal_ctx_override = cerbos_async_client.with_principal(
            principal_donald
        )
        have = await async_principal_ctx_override.check_resources(resource_list)
        _assert_check_resources_principal_override_with_output(have)

    async def test_plan_resources(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resourcedesc_leave_req: ResourceDesc,
    ):
        have = await async_principal_ctx.plan_resources(
            "view:*", resourcedesc_leave_req
        )
        _assert_plan_resources(have)


def _assert_check_resources(have: CheckResourcesResponse):
    assert have.failed() == False

    xx125 = have.get_resource(
        "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert xx125.is_allowed("view:public") == True
    assert xx125.is_allowed("approve") == False

    xx225 = have.get_resource("XX225")
    assert xx225 is not None
    assert xx225.is_allowed("view:public") == False
    assert xx225.is_allowed("approve") == False

    zz225 = have.get_resource("ZZ225")
    assert zz225 is None


def _assert_check_resources_empty_resources(have: CheckResourcesResponse):
    assert have.failed() == True
    assert have.status_msg.code == 3

    with pytest.raises(CerbosRequestException):
        have.raise_if_failed()


def _assert_plan_resources(have: PlanResourcesResponse):
    assert have.failed() == False
    assert have.resource_kind == "leave_request"
    assert have.policy_version == "20210210"
    assert have.filter.kind == PlanResourcesFilterKind.CONDITIONAL
    assert have.filter.condition is not None
    assert have.validation_errors is None


def _assert_plan_resources_validation(have: PlanResourcesResponse):
    assert have.failed() == False
    assert have.resource_kind == "leave_request"
    assert have.policy_version == "20210210"
    assert have.filter.kind == PlanResourcesFilterKind.ALWAYS_DENIED
    assert have.filter.condition is None
    assert len(have.validation_errors) == 2


def _assert_check_resources_with_output(have: CheckResourcesResponse):
    xx125 = have.get_resource(
        "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert len(xx125.outputs) == 1
    outputs = xx125.outputs[0].to_dict()
    assert outputs == {
        "src": "resource.leave_request.v20210210#public-view",
        "val": {
            "formatted_string": "id:john",
            "keys": "XX125",
            "pID": "john",
            "some_bool": True,
            "some_list": ["foo", "bar"],
            "something_nested": {
                "nested_bool": False,
                "nested_formatted_string": "id:john",
                "nested_list": ["nest_foo", 1.01],
                "nested_str": "foo",
            },
        },
    }


def _assert_check_resources_principal_override_with_output(
    have: CheckResourcesResponse,
):
    xx125 = have.get_resource(
        "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert len(xx125.outputs) == 1
    d = next(filter(lambda x: isinstance(x.val, dict), xx125.outputs))
    assert d.to_dict() == {
        "src": "resource.leave_request.v20210210#public-view",
        "val": {
            "formatted_string": "id:donald_duck",
            "keys": "XX125",
            "pID": "donald_duck",
            "some_bool": True,
            "some_list": ["foo", "bar"],
            "something_nested": {
                "nested_bool": False,
                "nested_formatted_string": "id:donald_duck",
                "nested_list": ["nest_foo", 1.01],
                "nested_str": "foo",
            },
        },
    }


@pytest.fixture
def principal_ctx(cerbos_client, principal_john):
    return cerbos_client.with_principal(principal_john)


@pytest.fixture
def async_principal_ctx(cerbos_async_client, principal_john):
    return cerbos_async_client.with_principal(principal_john)


@pytest.fixture
def resource_john_leave_req():
    return Resource(
        "XX125",
        "leave_request",
        policy_version="20210210",
        attr={
            "id": "XX125",
            "department": "marketing",
            "geography": "GB",
            "team": "design",
            "owner": "john",
        },
    )


@pytest.fixture
def resource_alice_leave_req():
    return Resource(
        "XX225",
        "leave_request",
        policy_version="20210210",
        attr={
            "id": "XX225",
            "department": "marketing",
            "geography": "GB",
            "team": "design",
            "owner": "alice",
            "status": "PENDING_APPROVAL",
        },
    )


@pytest.fixture
def resource_list(resource_john_leave_req, resource_alice_leave_req):
    return ResourceList(
        [
            ResourceAction(
                resource=resource_john_leave_req,
                actions={"view:public", "approve"},
            ),
            ResourceAction(
                resource=resource_alice_leave_req,
                actions={"view:private", "approve"},
            ),
        ]
    )


@pytest.fixture
def resourcedesc_leave_req():
    return ResourceDesc("leave_request", policy_version="20210210")


@pytest.fixture
def resourcedesc_leave_req_invalid():
    return ResourceDesc(
        "leave_request",
        policy_version="20210210",
        attr={"department": "accounting"},
    )


@pytest.fixture
def principal_john():
    return Principal(
        "john",
        roles={"employee"},
        policy_version="20210210",
        attr={"department": "marketing", "geography": "GB", "team": "design"},
    )


@pytest.fixture
def principal_maggie():
    return Principal(
        "maggie",
        roles={"manager"},
        policy_version="20210210",
        attr={
            "department": "marketing",
            "geography": "GB",
            "managed_geographies": "GB",
            "team": "design",
        },
    )


@pytest.fixture
def principal_donald():
    return Principal(
        "donald_duck",
        roles={"employee"},
        policy_version="20210210",
        attr={"department": "engineering", "geography": "GB", "team": "QA"},
    )


@pytest.fixture
def principal_maggie_invalid():
    return Principal(
        "maggie",
        roles={"manager"},
        policy_version="20210210",
        attr={
            "reader": False,
            "department": "accounting",
            "geography": "GB",
            "managed_geographies": "GB",
            "team": "design",
        },
    )



# Copyright 2021-2022 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import base64
import json
import os

import pytest

from cerbos.policy.v1 import policy_pb2
from cerbos.schema.v1 import schema_pb2
from cerbos.sdk.grpc.client import (
    AdminCredentials,
    AsyncCerbosAdminClient,
    CerbosAdminClient,
)

pytestmark = pytest.mark.anyio


schema_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "../store/policies/_schemas/principal.json",
)
with open(schema_path, "rb") as f:
    d = json.load(f)
    schema_contents = json.dumps(d, separators=(",", ":")).encode()


class TestCerbosAdminClient:
    def test_admin_credentials_metadata(self, cerbos_admin_client: CerbosAdminClient):
        admin_credentials = AdminCredentials(username="user", password="password")
        metadata = admin_credentials.metadata()
        encoded_credentials = base64.b64encode("user:password".encode("utf-8")).decode(
            "utf-8"
        )
        expected_metadata = (("authorization", f"Basic {encoded_credentials}"),)
        assert metadata == expected_metadata

    def test_add_or_update_policy(self, cerbos_admin_client: CerbosAdminClient):
        cerbos_admin_client.add_or_update_policy(
            [
                policy_pb2.Policy(
                    api_version="api.cerbos.dev/v1",
                    principal_policy=policy_pb2.PrincipalPolicy(
                        principal="terry", version="default"
                    ),
                )
            ]
        )

    def test_list_policies(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.list_policies()
        assert resp.policy_ids == ["principal.terry.vdefault"]

    def test_get_policy(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.get_policy(["principal.terry.vdefault"])
        assert len(resp.policies) == 1
        p = resp.policies[0]
        assert p.principal_policy.principal == "terry"
        assert p.principal_policy.version == "default"

    def test_disable_policy(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.disable_policy(["principal.terry.vdefault"])
        assert resp.disabled_policies == 1

    def test_enable_policy(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.enable_policy(["principal.terry.vdefault"])
        assert resp.enabled_policies == 1

    def test_add_or_update_schema(self, cerbos_admin_client: CerbosAdminClient):
        cerbos_admin_client.add_or_update_schema(
            [
                schema_pb2.Schema(
                    id="principal.json",
                    definition=schema_contents,
                )
            ]
        )

    def test_list_schemas(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.list_schemas()
        assert len(resp.schema_ids) == 1
        assert resp.schema_ids[0] == "principal.json"

    def test_get_schema(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.get_schema(["principal.json"])
        assert len(resp.schemas) == 1
        s = resp.schemas[0]
        assert s.id == "principal.json"
        assert s.definition == schema_contents

    def test_delete_schema(self, cerbos_admin_client: CerbosAdminClient):
        resp = cerbos_admin_client.delete_schema(["principal.json"])
        assert resp.deleted_schemas == 1

    def test_reload_store(self, cerbos_admin_client: CerbosAdminClient):
        cerbos_admin_client.reload_store()


class TestAsyncCerbosAdminClient:
    async def test_add_or_update_policy(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        await cerbos_async_admin_client.add_or_update_policy(
            [
                policy_pb2.Policy(
                    api_version="api.cerbos.dev/v1",
                    principal_policy=policy_pb2.PrincipalPolicy(
                        principal="terry", version="default"
                    ),
                )
            ]
        )

    async def test_list_policies(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        resp = await cerbos_async_admin_client.list_policies()
        assert resp.policy_ids == ["principal.terry.vdefault"]

    async def test_get_policy(self, cerbos_async_admin_client: AsyncCerbosAdminClient):
        resp = await cerbos_async_admin_client.get_policy(["principal.terry.vdefault"])
        assert len(resp.policies) == 1
        p = resp.policies[0]
        assert p.principal_policy.principal == "terry"
        assert p.principal_policy.version == "default"

    async def test_disable_policy(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        resp = await cerbos_async_admin_client.disable_policy(
            ["principal.terry.vdefault"]
        )
        assert resp.disabled_policies == 1

    async def test_enable_policy(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        resp = await cerbos_async_admin_client.enable_policy(
            ["principal.terry.vdefault"]
        )
        assert resp.enabled_policies == 1

    async def test_add_or_update_schema(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        await cerbos_async_admin_client.add_or_update_schema(
            [
                schema_pb2.Schema(
                    id="principal.json",
                    definition=schema_contents,
                )
            ]
        )

    async def test_list_schemas(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        resp = await cerbos_async_admin_client.list_schemas()
        assert len(resp.schema_ids) == 1
        assert resp.schema_ids[0] == "principal.json"

    async def test_get_schema(self, cerbos_async_admin_client: AsyncCerbosAdminClient):
        resp = await cerbos_async_admin_client.get_schema(["principal.json"])
        assert len(resp.schemas) == 1
        s = resp.schemas[0]
        assert s.id == "principal.json"
        assert s.definition == schema_contents

    async def test_delete_schema(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        resp = await cerbos_async_admin_client.delete_schema(["principal.json"])
        assert resp.deleted_schemas == 1

    async def test_reload_store(
        self, cerbos_async_admin_client: AsyncCerbosAdminClient
    ):
        await cerbos_async_admin_client.reload_store()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cerbos/audit/v1/audit.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cerbos.engine.v1 import engine_pb2 as cerbos_dot_engine_dot_v1_dot_engine__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x63\x65rbos/audit/v1/audit.proto\x12\x0f\x63\x65rbos.audit.v1\x1a\x1d\x63\x65rbos/engine/v1/engine.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xec\x02\n\x0e\x41\x63\x63\x65ssLogEntry\x12\x17\n\x07\x63\x61ll_id\x18\x01 \x01(\tR\x06\x63\x61llId\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12)\n\x04peer\x18\x03 \x01(\x0b\x32\x15.cerbos.audit.v1.PeerR\x04peer\x12I\n\x08metadata\x18\x04 \x03(\x0b\x32-.cerbos.audit.v1.AccessLogEntry.MetadataEntryR\x08metadata\x12\x16\n\x06method\x18\x05 \x01(\tR\x06method\x12\x1f\n\x0bstatus_code\x18\x06 \x01(\rR\nstatusCode\x1aX\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.cerbos.audit.v1.MetaValuesR\x05value:\x02\x38\x01\"\xb8\x07\n\x10\x44\x65\x63isionLogEntry\x12\x17\n\x07\x63\x61ll_id\x18\x01 \x01(\tR\x06\x63\x61llId\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12)\n\x04peer\x18\x03 \x01(\x0b\x32\x15.cerbos.audit.v1.PeerR\x04peer\x12\x34\n\x06inputs\x18\x04 \x03(\x0b\x32\x1c.cerbos.engine.v1.CheckInputR\x06inputs\x12\x37\n\x07outputs\x18\x05 \x03(\x0b\x32\x1d.cerbos.engine.v1.CheckOutputR\x07outputs\x12\x14\n\x05\x65rror\x18\x06 \x01(\tR\x05\x65rror\x12[\n\x0f\x63heck_resources\x18\x07 \x01(\x0b\x32\x30.cerbos.audit.v1.DecisionLogEntry.CheckResourcesH\x00R\x0e\x63heckResources\x12X\n\x0eplan_resources\x18\x08 \x01(\x0b\x32/.cerbos.audit.v1.DecisionLogEntry.PlanResourcesH\x00R\rplanResources\x12K\n\x08metadata\x18\x0f \x03(\x0b\x32/.cerbos.audit.v1.DecisionLogEntry.MetadataEntryR\x08metadata\x1a\x95\x01\n\x0e\x43heckResources\x12\x34\n\x06inputs\x18\x01 \x03(\x0b\x32\x1c.cerbos.engine.v1.CheckInputR\x06inputs\x12\x37\n\x07outputs\x18\x02 \x03(\x0b\x32\x1d.cerbos.engine.v1.CheckOutputR\x07outputs\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rror\x1a\xa0\x01\n\rPlanResources\x12:\n\x05input\x18\x01 \x01(\x0b\x32$.cerbos.engine.v1.PlanResourcesInputR\x05input\x12=\n\x06output\x18\x02 \x01(\x0b\x32%.cerbos.engine.v1.PlanResourcesOutputR\x06output\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rror\x1aX\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.cerbos.audit.v1.MetaValuesR\x05value:\x02\x38\x01\x42\x08\n\x06method\"$\n\nMetaValues\x12\x16\n\x06values\x18\x01 \x03(\tR\x06values\"\x81\x01\n\x04Peer\x12\x18\n\x07\x61\x64\x64ress\x18\x01 \x01(\tR\x07\x61\x64\x64ress\x12\x1b\n\tauth_info\x18\x02 \x01(\tR\x08\x61uthInfo\x12\x1d\n\nuser_agent\x18\x03 \x01(\tR\tuserAgent\x12#\n\rforwarded_for\x18\x04 \x01(\tR\x0c\x66orwardedForBk\n\x17\x64\x65v.cerbos.api.v1.auditZ:github.com/cerbos/cerbos/api/genpb/cerbos/audit/v1;auditv1\xaa\x02\x13\x43\x65rbos.Api.V1.Auditb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.audit.v1.audit_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027dev.cerbos.api.v1.auditZ:github.com/cerbos/cerbos/api/genpb/cerbos/audit/v1;auditv1\252\002\023Cerbos.Api.V1.Audit'
  _ACCESSLOGENTRY_METADATAENTRY._options = None
  _ACCESSLOGENTRY_METADATAENTRY._serialized_options = b'8\001'
  _DECISIONLOGENTRY_METADATAENTRY._options = None
  _DECISIONLOGENTRY_METADATAENTRY._serialized_options = b'8\001'
  _globals['_ACCESSLOGENTRY']._serialized_start=113
  _globals['_ACCESSLOGENTRY']._serialized_end=477
  _globals['_ACCESSLOGENTRY_METADATAENTRY']._serialized_start=389
  _globals['_ACCESSLOGENTRY_METADATAENTRY']._serialized_end=477
  _globals['_DECISIONLOGENTRY']._serialized_start=480
  _globals['_DECISIONLOGENTRY']._serialized_end=1432
  _globals['_DECISIONLOGENTRY_CHECKRESOURCES']._serialized_start=1020
  _globals['_DECISIONLOGENTRY_CHECKRESOURCES']._serialized_end=1169
  _globals['_DECISIONLOGENTRY_PLANRESOURCES']._serialized_start=1172
  _globals['_DECISIONLOGENTRY_PLANRESOURCES']._serialized_end=1332
  _globals['_DECISIONLOGENTRY_METADATAENTRY']._serialized_start=389
  _globals['_DECISIONLOGENTRY_METADATAENTRY']._serialized_end=477
  _globals['_METAVALUES']._serialized_start=1434
  _globals['_METAVALUES']._serialized_end=1470
  _globals['_PEER']._serialized_start=1473
  _globals['_PEER']._serialized_end=1602
# @@protoc_insertion_point(module_scope)



# Copyright 2021-2022 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

from typing import List

import grpc
import pytest
from google.protobuf import struct_pb2
from google.protobuf.json_format import MessageToDict

from cerbos.engine.v1 import engine_pb2
from cerbos.request.v1 import request_pb2
from cerbos.response.v1 import response_pb2
from cerbos.sdk.grpc.client import (
    AsyncCerbosClient,
    AsyncPrincipalContext,
    CerbosClient,
    PrincipalContext,
)
from cerbos.sdk.grpc.utils import get_resource, is_allowed

pytestmark = pytest.mark.anyio


class TestCerbosClient:
    def test_is_allowed(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_john: engine_pb2.Principal,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = cerbos_grpc_client.is_allowed(
            "view:public", principal_john, resource_john_leave_req
        )
        assert have

    def test_principal_context_is_allowed(
        self,
        principal_ctx: PrincipalContext,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = principal_ctx.is_allowed("view:public", resource_john_leave_req)
        assert have

    def test_check_resources(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_john: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = cerbos_grpc_client.check_resources(principal_john, resource_list)
        _assert_check_resources(have)

    def test_check_resources_empty_resources(
        self, cerbos_grpc_client: CerbosClient, principal_john: engine_pb2.Principal
    ):
        with pytest.raises(
            grpc.RpcError,
            match="invalid CheckResourcesRequest.Resources: value must contain at least 1 item\\(s\\)",
        ) as e:
            cerbos_grpc_client.check_resources(principal_john, [])
        err = e.value
        assert err.code().value[0] == 3

    def test_plan_resources(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_maggie: engine_pb2.Principal,
        resourcedesc_leave_req: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = cerbos_grpc_client.plan_resources(
            "approve", principal_maggie, resourcedesc_leave_req
        )
        _assert_plan_resources(have)

    def test_plan_resources_validation(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_maggie_invalid: engine_pb2.Principal,
        resourcedesc_leave_req_invalid: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = cerbos_grpc_client.plan_resources(
            "approve", principal_maggie_invalid, resourcedesc_leave_req_invalid
        )
        _assert_plan_resources_validation(have)

    def test_check_resources_with_output(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_john: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = cerbos_grpc_client.check_resources(principal_john, resource_list)
        _assert_check_resources_with_output(have)

    def test_is_allowed_with_export_variables(
        self,
        cerbos_grpc_client: CerbosClient,
    ):
        p = engine_pb2.Principal(
            id="daffy",
            roles={"user"},
            policy_version="default",
            attr={
                "vip": struct_pb2.Value(bool_value=True),
            },
        )
        r = engine_pb2.Resource(
            id="XX325",
            kind="album:object",
            policy_version="default",
        )
        have = cerbos_grpc_client.is_allowed(
            "view-as-vip",
            p,
            r,
        )
        assert have


class TestPrincipalContext:
    def test_is_allowed(
        self,
        principal_ctx: PrincipalContext,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = principal_ctx.is_allowed("view:public", resource_john_leave_req)
        assert have

    def test_check_resources(
        self,
        principal_ctx: PrincipalContext,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = principal_ctx.check_resources(resource_list)
        _assert_check_resources(have)

    def test_check_resources_with_output(
        self,
        cerbos_grpc_client: CerbosClient,
        principal_donald: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        principal_ctx_override = cerbos_grpc_client.with_principal(principal_donald)
        have = principal_ctx_override.check_resources(resource_list)
        _assert_check_resources_principal_override_with_output(have)

    def test_plan_resources(
        self,
        principal_ctx: PrincipalContext,
        resourcedesc_leave_req: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = principal_ctx.plan_resources("view:*", resourcedesc_leave_req)
        _assert_plan_resources(have)


class TestAsyncCerbosClient:
    async def test_is_allowed(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_john: engine_pb2.Principal,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = await cerbos_async_grpc_client.is_allowed(
            "view:public", principal_john, resource_john_leave_req
        )
        assert have

    async def test_principal_context_is_allowed(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = await async_principal_ctx.is_allowed(
            "view:public", resource_john_leave_req
        )
        assert have

    async def test_check_resources(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_john: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = await cerbos_async_grpc_client.check_resources(
            principal_john, resource_list
        )
        _assert_check_resources(have)

    async def test_check_resources_empty_resources(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_john: engine_pb2.Principal,
    ):
        with pytest.raises(
            grpc.aio.AioRpcError,
            match="invalid CheckResourcesRequest.Resources: value must contain at least 1 item\\(s\\)",
        ) as e:
            await cerbos_async_grpc_client.check_resources(principal_john, [])
        err = e.value
        assert err.code().value[0] == 3

    async def test_plan_resources(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_maggie: engine_pb2.Principal,
        resourcedesc_leave_req: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = await cerbos_async_grpc_client.plan_resources(
            "approve", principal_maggie, resourcedesc_leave_req
        )
        _assert_plan_resources(have)

    async def test_plan_resources_validation(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_maggie_invalid: engine_pb2.Principal,
        resourcedesc_leave_req_invalid: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = await cerbos_async_grpc_client.plan_resources(
            "approve", principal_maggie_invalid, resourcedesc_leave_req_invalid
        )
        _assert_plan_resources_validation(have)

    async def test_check_resources_with_output(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_john: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = await cerbos_async_grpc_client.check_resources(
            principal_john, resource_list
        )
        _assert_check_resources_with_output(have)


class TestAsyncAsyncPrincipalContext:
    async def test_is_allowed(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resource_john_leave_req: engine_pb2.Resource,
    ):
        have = await async_principal_ctx.is_allowed(
            "view:public", resource_john_leave_req
        )
        assert have

    async def test_check_resources(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        have = await async_principal_ctx.check_resources(resource_list)
        _assert_check_resources(have)

    async def test_check_resources_with_output(
        self,
        cerbos_async_grpc_client: AsyncCerbosClient,
        principal_donald: engine_pb2.Principal,
        resource_list: List[request_pb2.CheckResourcesRequest.ResourceEntry],
    ):
        async_principal_ctx_override = cerbos_async_grpc_client.with_principal(
            principal_donald
        )
        have = await async_principal_ctx_override.check_resources(resource_list)
        _assert_check_resources_principal_override_with_output(have)

    async def test_plan_resources(
        self,
        async_principal_ctx: AsyncPrincipalContext,
        resourcedesc_leave_req: engine_pb2.PlanResourcesInput.Resource,
    ):
        have = await async_principal_ctx.plan_resources(
            "view:*", resourcedesc_leave_req
        )
        _assert_plan_resources(have)


def _assert_check_resources(have: response_pb2.CheckResourcesResponse):
    xx125 = get_resource(
        have, "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert is_allowed(xx125, "view:public")
    assert not is_allowed(xx125, "approve")

    xx225 = get_resource(have, "XX225")
    assert xx225 is not None
    assert not is_allowed(xx225, "view:public")
    assert not is_allowed(xx225, "approve")

    zz225 = get_resource(have, "ZZ225")
    assert zz225 is None


def _assert_plan_resources(have: response_pb2.PlanResourcesResponse):
    assert have.resource_kind == "leave_request"
    assert have.policy_version == "20210210"
    assert have.filter.kind == engine_pb2.PlanResourcesFilter.KIND_CONDITIONAL
    assert have.filter.condition is not None
    assert len(have.validation_errors) == 0


def _assert_plan_resources_validation(have: response_pb2.PlanResourcesResponse):
    assert have.resource_kind == "leave_request"
    assert have.policy_version == "20210210"
    assert have.filter.kind == engine_pb2.PlanResourcesFilter.KIND_ALWAYS_DENIED
    assert len(have.filter.condition.SerializeToString()) == 0
    assert len(have.validation_errors) == 2


def _assert_check_resources_with_output(have: response_pb2.CheckResourcesResponse):
    xx125 = get_resource(
        have, "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert len(xx125.outputs) == 1
    outputs = MessageToDict(xx125.outputs[0])
    assert outputs == {
        "src": "resource.leave_request.v20210210#public-view",
        "val": {
            "formatted_string": "id:john",
            "keys": "XX125",
            "pID": "john",
            "some_bool": True,
            "some_list": ["foo", "bar"],
            "something_nested": {
                "nested_bool": False,
                "nested_formatted_string": "id:john",
                "nested_list": ["nest_foo", 1.01],
                "nested_str": "foo",
            },
        },
    }


def _assert_check_resources_principal_override_with_output(
    have: response_pb2.CheckResourcesResponse,
):
    xx125 = get_resource(
        have, "XX125", predicate=lambda r: r.policy_version == "20210210"
    )
    assert xx125 is not None
    assert len(xx125.outputs) == 1
    d = next(filter(lambda x: x.val.HasField("struct_value"), xx125.outputs))
    assert MessageToDict(d) == {
        "src": "resource.leave_request.v20210210#public-view",
        "val": {
            "formatted_string": "id:donald_duck",
            "keys": "XX125",
            "pID": "donald_duck",
            "some_bool": True,
            "some_list": ["foo", "bar"],
            "something_nested": {
                "nested_bool": False,
                "nested_formatted_string": "id:donald_duck",
                "nested_list": ["nest_foo", 1.01],
                "nested_str": "foo",
            },
        },
    }


@pytest.fixture
def principal_ctx(cerbos_grpc_client, principal_john):
    return cerbos_grpc_client.with_principal(principal_john)


@pytest.fixture
def async_principal_ctx(cerbos_async_grpc_client, principal_john):
    return cerbos_async_grpc_client.with_principal(principal_john)


@pytest.fixture
def resource_john_leave_req():
    return engine_pb2.Resource(
        id="XX125",
        kind="leave_request",
        policy_version="20210210",
        attr={
            "id": struct_pb2.Value(string_value="XX125"),
            "department": struct_pb2.Value(string_value="marketing"),
            "geography": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="design"),
            "owner": struct_pb2.Value(string_value="john"),
        },
    )


@pytest.fixture
def resource_alice_leave_req():
    return engine_pb2.Resource(
        id="XX225",
        kind="leave_request",
        policy_version="20210210",
        attr={
            "id": struct_pb2.Value(string_value="XX225"),
            "department": struct_pb2.Value(string_value="marketing"),
            "geography": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="design"),
            "owner": struct_pb2.Value(string_value="alice"),
            "status": struct_pb2.Value(string_value="PENDING_APPROVAL"),
        },
    )


@pytest.fixture
def resource_list(resource_john_leave_req, resource_alice_leave_req):
    return [
        request_pb2.CheckResourcesRequest.ResourceEntry(
            resource=resource_john_leave_req,
            actions={"view:public", "approve"},
        ),
        request_pb2.CheckResourcesRequest.ResourceEntry(
            resource=resource_alice_leave_req,
            actions={"view:private", "approve"},
        ),
    ]


@pytest.fixture
def resourcedesc_leave_req():
    return engine_pb2.PlanResourcesInput.Resource(
        kind="leave_request",
        policy_version="20210210",
    )


@pytest.fixture
def resourcedesc_leave_req_invalid():
    return engine_pb2.PlanResourcesInput.Resource(
        kind="leave_request",
        policy_version="20210210",
        attr={"department": struct_pb2.Value(string_value="accounting")},
    )


@pytest.fixture
def principal_john():
    return engine_pb2.Principal(
        id="john",
        roles=["employee"],
        policy_version="20210210",
        attr={
            "department": struct_pb2.Value(string_value="marketing"),
            "geography": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="design"),
        },
    )


@pytest.fixture
def principal_maggie():
    return engine_pb2.Principal(
        id="maggie",
        roles={"manager"},
        policy_version="20210210",
        attr={
            "department": struct_pb2.Value(string_value="marketing"),
            "geography": struct_pb2.Value(string_value="GB"),
            "managed_geographies": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="design"),
        },
    )


@pytest.fixture
def principal_donald():
    return engine_pb2.Principal(
        id="donald_duck",
        roles={"employee"},
        policy_version="20210210",
        attr={
            "department": struct_pb2.Value(string_value="engineering"),
            "geography": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="QA"),
        },
    )


@pytest.fixture
def principal_maggie_invalid():
    return engine_pb2.Principal(
        id="maggie",
        roles={"manager"},
        policy_version="20210210",
        attr={
            "reader": struct_pb2.Value(bool_value=False),
            "department": struct_pb2.Value(string_value="accounting"),
            "geography": struct_pb2.Value(string_value="GB"),
            "managed_geographies": struct_pb2.Value(string_value="GB"),
            "team": struct_pb2.Value(string_value="design"),
        },
    )



# Copyright 2021-2022 Zenauth Ltd.
# SPDX-License-Identifier: Apache-2.0

import logging
import os

import pytest

from cerbos.sdk.client import AsyncCerbosClient, CerbosClient
from cerbos.sdk.container import CerbosContainer
from cerbos.sdk.grpc.client import AdminCredentials, AsyncCerbosAdminClient
from cerbos.sdk.grpc.client import AsyncCerbosClient as AsyncGrpcCerbosClient
from cerbos.sdk.grpc.client import CerbosAdminClient
from cerbos.sdk.grpc.client import CerbosClient as GrpcCerbosClient
from cerbos.sdk.model import *


@pytest.fixture(scope="module")
def anyio_backend():
    return "asyncio"


_params = [("http"), ("uds")]


@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=http,listener={p}" for p in _params],
)
def cerbos_client(request, tmp_path_factory):
    container, host = start_container("http", request.param, tmp_path_factory)
    if container:
        with CerbosClient(host, debug=True) as client:
            yield client
        container.stop()


@pytest.mark.anyio
@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=http,listener={p}" for p in _params],
)
async def cerbos_async_client(anyio_backend, request, tmp_path_factory):
    container, host = start_container("http", request.param, tmp_path_factory)
    if container:
        async with AsyncCerbosClient(host, debug=True) as client:
            yield client
        container.stop()


@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=grpc,listener={p}" for p in _params],
)
def cerbos_grpc_client(request, tmp_path_factory):
    container, host = start_container("grpc", request.param, tmp_path_factory)
    if container:
        with GrpcCerbosClient(host) as client:
            yield client
        container.stop()


@pytest.mark.anyio
@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=grpc,listener={p}" for p in _params],
)
async def cerbos_async_grpc_client(anyio_backend, request, tmp_path_factory):
    container, host = start_container("grpc", request.param, tmp_path_factory)
    if container:
        async with AsyncGrpcCerbosClient(host) as client:
            yield client
        container.stop()


@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=grpc,listener={p}" for p in _params],
)
def cerbos_admin_client(request, tmp_path_factory):
    container, host = start_container(
        "grpc", request.param, tmp_path_factory, with_admin=True
    )
    if container:
        with CerbosAdminClient(host, AdminCredentials(password="randomHash")) as client:
            yield client
        container.stop()


@pytest.mark.anyio
@pytest.fixture(
    scope="module",
    params=_params,
    ids=[f"transport=grpc,listener={p}" for p in _params],
)
async def cerbos_async_admin_client(anyio_backend, request, tmp_path_factory):
    container, host = start_container(
        "grpc", request.param, tmp_path_factory, with_admin=True
    )
    if container:
        async with AsyncCerbosAdminClient(
            host, AdminCredentials(password="randomHash")
        ) as client:
            yield client
        container.stop()


def start_container(client_type, listener, tmp_path_factory, with_admin=False):
    store_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "store")
    container = CerbosContainer(image="ghcr.io/cerbos/cerbos:dev")
    container.with_volume_mapping(store_dir, "/store")
    container.with_env("CERBOS_NO_TELEMETRY", "1")

    with_mutable_store = ""
    if with_admin:
        with_mutable_store = (
            " --set=storage.driver=sqlite3 --set=storage.sqlite3.dsn=:memory:"
        )

    if listener == "http":
        container.with_command(
            "server --config=./store/conf.yaml --set=schema.enforcement=reject"
            + with_mutable_store
        )
        container.start()
        container.wait_until_ready()

        host = container.http_host() if client_type == "http" else container.grpc_host()
    else:
        # (07-23 saml) macOS+docker does not play nice when it comes to sharing UDS across the host and container. I've not figured out how to work around
        # this yet so I tend to comment out `uds` in the fixture params above and rely on CI to test the full suite.
        sock_dir = tmp_path_factory.mktemp("socket")

        container.with_volume_mapping(sock_dir, "/socket", "rw")
        container.with_command(
            f"server --config=./store/conf.yaml --set=server.{client_type}ListenAddr=unix:/socket/cerbos.{client_type} --set=server.udsFileMode=0o777 --set=schema.enforcement=reject"
            + with_mutable_store
        )
        container.start()
        container.wait_until_ready()

        host = f"unix:{sock_dir}/cerbos.{client_type}"

    logging.info(f"Cerbos Address: {host}")
    return container, host



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cerbos/schema/v1/schema.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x63\x65rbos/schema/v1/schema.proto\x12\x10\x63\x65rbos.schema.v1\x1a\x1fgoogle/api/field_behavior.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a\x17validate/validate.proto\"\xce\x01\n\x0fValidationError\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12@\n\x06source\x18\x03 \x01(\x0e\x32(.cerbos.schema.v1.ValidationError.SourceR\x06source\"K\n\x06Source\x12\x16\n\x12SOURCE_UNSPECIFIED\x10\x00\x12\x14\n\x10SOURCE_PRINCIPAL\x10\x01\x12\x13\n\x0fSOURCE_RESOURCE\x10\x02\"\xcb\x01\n\x06Schema\x12U\n\x02id\x18\x01 \x01(\tBE\x92\x41\x34\x32 Unique identifier for the schemaJ\x10\"principal.json\"\xe2\x41\x01\x02\xfa\x42\x07r\x05\x10\x01\x18\xff\x01R\x02id\x12j\n\ndefinition\x18\x02 \x01(\x0c\x42J\x92\x41<2\x16JSON schema definitionJ\"{\"type\":\"object\", \"properties\":{}}\xe2\x41\x01\x02\xfa\x42\x04z\x02\x10\nR\ndefinitionBo\n\x18\x64\x65v.cerbos.api.v1.schemaZ<github.com/cerbos/cerbos/api/genpb/cerbos/schema/v1;schemav1\xaa\x02\x14\x43\x65rbos.Api.V1.Schemab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.schema.v1.schema_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030dev.cerbos.api.v1.schemaZ<github.com/cerbos/cerbos/api/genpb/cerbos/schema/v1;schemav1\252\002\024Cerbos.Api.V1.Schema'
  _SCHEMA.fields_by_name['id']._options = None
  _SCHEMA.fields_by_name['id']._serialized_options = b'\222A42 Unique identifier for the schemaJ\020\"principal.json\"\342A\001\002\372B\007r\005\020\001\030\377\001'
  _SCHEMA.fields_by_name['definition']._options = None
  _SCHEMA.fields_by_name['definition']._serialized_options = b'\222A<2\026JSON schema definitionJ\"{\"type\":\"object\", \"properties\":{}}\342A\001\002\372B\004z\002\020\n'
  _globals['_VALIDATIONERROR']._serialized_start=158
  _globals['_VALIDATIONERROR']._serialized_end=364
  _globals['_VALIDATIONERROR_SOURCE']._serialized_start=289
  _globals['_VALIDATIONERROR_SOURCE']._serialized_end=364
  _globals['_SCHEMA']._serialized_start=367
  _globals['_SCHEMA']._serialized_end=570
# @@protoc_insertion_point(module_scope)



from cerbos.audit.v1 import audit_pb2 as _audit_pb2
from cerbos.effect.v1 import effect_pb2 as _effect_pb2
from cerbos.engine.v1 import engine_pb2 as _engine_pb2
from cerbos.policy.v1 import policy_pb2 as _policy_pb2
from cerbos.schema.v1 import schema_pb2 as _schema_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PlanResourcesResponse(_message.Message):
    __slots__ = ["request_id", "action", "resource_kind", "policy_version", "filter", "meta", "validation_errors"]
    class Meta(_message.Message):
        __slots__ = ["filter_debug", "matched_scope"]
        FILTER_DEBUG_FIELD_NUMBER: _ClassVar[int]
        MATCHED_SCOPE_FIELD_NUMBER: _ClassVar[int]
        filter_debug: str
        matched_scope: str
        def __init__(self, filter_debug: _Optional[str] = ..., matched_scope: _Optional[str] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_KIND_FIELD_NUMBER: _ClassVar[int]
    POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    action: str
    resource_kind: str
    policy_version: str
    filter: _engine_pb2.PlanResourcesFilter
    meta: PlanResourcesResponse.Meta
    validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
    def __init__(self, request_id: _Optional[str] = ..., action: _Optional[str] = ..., resource_kind: _Optional[str] = ..., policy_version: _Optional[str] = ..., filter: _Optional[_Union[_engine_pb2.PlanResourcesFilter, _Mapping]] = ..., meta: _Optional[_Union[PlanResourcesResponse.Meta, _Mapping]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ...) -> None: ...

class CheckResourceSetResponse(_message.Message):
    __slots__ = ["request_id", "resource_instances", "meta"]
    class ActionEffectMap(_message.Message):
        __slots__ = ["actions", "validation_errors"]
        class ActionsEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: _effect_pb2.Effect
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_effect_pb2.Effect, str]] = ...) -> None: ...
        ACTIONS_FIELD_NUMBER: _ClassVar[int]
        VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
        actions: _containers.ScalarMap[str, _effect_pb2.Effect]
        validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
        def __init__(self, actions: _Optional[_Mapping[str, _effect_pb2.Effect]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ...) -> None: ...
    class Meta(_message.Message):
        __slots__ = ["resource_instances"]
        class EffectMeta(_message.Message):
            __slots__ = ["matched_policy", "matched_scope"]
            MATCHED_POLICY_FIELD_NUMBER: _ClassVar[int]
            MATCHED_SCOPE_FIELD_NUMBER: _ClassVar[int]
            matched_policy: str
            matched_scope: str
            def __init__(self, matched_policy: _Optional[str] = ..., matched_scope: _Optional[str] = ...) -> None: ...
        class ActionMeta(_message.Message):
            __slots__ = ["actions", "effective_derived_roles"]
            class ActionsEntry(_message.Message):
                __slots__ = ["key", "value"]
                KEY_FIELD_NUMBER: _ClassVar[int]
                VALUE_FIELD_NUMBER: _ClassVar[int]
                key: str
                value: CheckResourceSetResponse.Meta.EffectMeta
                def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[CheckResourceSetResponse.Meta.EffectMeta, _Mapping]] = ...) -> None: ...
            ACTIONS_FIELD_NUMBER: _ClassVar[int]
            EFFECTIVE_DERIVED_ROLES_FIELD_NUMBER: _ClassVar[int]
            actions: _containers.MessageMap[str, CheckResourceSetResponse.Meta.EffectMeta]
            effective_derived_roles: _containers.RepeatedScalarFieldContainer[str]
            def __init__(self, actions: _Optional[_Mapping[str, CheckResourceSetResponse.Meta.EffectMeta]] = ..., effective_derived_roles: _Optional[_Iterable[str]] = ...) -> None: ...
        class ResourceInstancesEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: CheckResourceSetResponse.Meta.ActionMeta
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[CheckResourceSetResponse.Meta.ActionMeta, _Mapping]] = ...) -> None: ...
        RESOURCE_INSTANCES_FIELD_NUMBER: _ClassVar[int]
        resource_instances: _containers.MessageMap[str, CheckResourceSetResponse.Meta.ActionMeta]
        def __init__(self, resource_instances: _Optional[_Mapping[str, CheckResourceSetResponse.Meta.ActionMeta]] = ...) -> None: ...
    class ResourceInstancesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: CheckResourceSetResponse.ActionEffectMap
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[CheckResourceSetResponse.ActionEffectMap, _Mapping]] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_INSTANCES_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    resource_instances: _containers.MessageMap[str, CheckResourceSetResponse.ActionEffectMap]
    meta: CheckResourceSetResponse.Meta
    def __init__(self, request_id: _Optional[str] = ..., resource_instances: _Optional[_Mapping[str, CheckResourceSetResponse.ActionEffectMap]] = ..., meta: _Optional[_Union[CheckResourceSetResponse.Meta, _Mapping]] = ...) -> None: ...

class CheckResourceBatchResponse(_message.Message):
    __slots__ = ["request_id", "results"]
    class ActionEffectMap(_message.Message):
        __slots__ = ["resource_id", "actions", "validation_errors"]
        class ActionsEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: _effect_pb2.Effect
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_effect_pb2.Effect, str]] = ...) -> None: ...
        RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
        ACTIONS_FIELD_NUMBER: _ClassVar[int]
        VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
        resource_id: str
        actions: _containers.ScalarMap[str, _effect_pb2.Effect]
        validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
        def __init__(self, resource_id: _Optional[str] = ..., actions: _Optional[_Mapping[str, _effect_pb2.Effect]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    results: _containers.RepeatedCompositeFieldContainer[CheckResourceBatchResponse.ActionEffectMap]
    def __init__(self, request_id: _Optional[str] = ..., results: _Optional[_Iterable[_Union[CheckResourceBatchResponse.ActionEffectMap, _Mapping]]] = ...) -> None: ...

class CheckResourcesResponse(_message.Message):
    __slots__ = ["request_id", "results"]
    class ResultEntry(_message.Message):
        __slots__ = ["resource", "actions", "validation_errors", "meta", "outputs"]
        class Resource(_message.Message):
            __slots__ = ["id", "kind", "policy_version", "scope"]
            ID_FIELD_NUMBER: _ClassVar[int]
            KIND_FIELD_NUMBER: _ClassVar[int]
            POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
            SCOPE_FIELD_NUMBER: _ClassVar[int]
            id: str
            kind: str
            policy_version: str
            scope: str
            def __init__(self, id: _Optional[str] = ..., kind: _Optional[str] = ..., policy_version: _Optional[str] = ..., scope: _Optional[str] = ...) -> None: ...
        class Meta(_message.Message):
            __slots__ = ["actions", "effective_derived_roles"]
            class EffectMeta(_message.Message):
                __slots__ = ["matched_policy", "matched_scope"]
                MATCHED_POLICY_FIELD_NUMBER: _ClassVar[int]
                MATCHED_SCOPE_FIELD_NUMBER: _ClassVar[int]
                matched_policy: str
                matched_scope: str
                def __init__(self, matched_policy: _Optional[str] = ..., matched_scope: _Optional[str] = ...) -> None: ...
            class ActionsEntry(_message.Message):
                __slots__ = ["key", "value"]
                KEY_FIELD_NUMBER: _ClassVar[int]
                VALUE_FIELD_NUMBER: _ClassVar[int]
                key: str
                value: CheckResourcesResponse.ResultEntry.Meta.EffectMeta
                def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[CheckResourcesResponse.ResultEntry.Meta.EffectMeta, _Mapping]] = ...) -> None: ...
            ACTIONS_FIELD_NUMBER: _ClassVar[int]
            EFFECTIVE_DERIVED_ROLES_FIELD_NUMBER: _ClassVar[int]
            actions: _containers.MessageMap[str, CheckResourcesResponse.ResultEntry.Meta.EffectMeta]
            effective_derived_roles: _containers.RepeatedScalarFieldContainer[str]
            def __init__(self, actions: _Optional[_Mapping[str, CheckResourcesResponse.ResultEntry.Meta.EffectMeta]] = ..., effective_derived_roles: _Optional[_Iterable[str]] = ...) -> None: ...
        class ActionsEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: _effect_pb2.Effect
            def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_effect_pb2.Effect, str]] = ...) -> None: ...
        RESOURCE_FIELD_NUMBER: _ClassVar[int]
        ACTIONS_FIELD_NUMBER: _ClassVar[int]
        VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
        META_FIELD_NUMBER: _ClassVar[int]
        OUTPUTS_FIELD_NUMBER: _ClassVar[int]
        resource: CheckResourcesResponse.ResultEntry.Resource
        actions: _containers.ScalarMap[str, _effect_pb2.Effect]
        validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
        meta: CheckResourcesResponse.ResultEntry.Meta
        outputs: _containers.RepeatedCompositeFieldContainer[_engine_pb2.OutputEntry]
        def __init__(self, resource: _Optional[_Union[CheckResourcesResponse.ResultEntry.Resource, _Mapping]] = ..., actions: _Optional[_Mapping[str, _effect_pb2.Effect]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ..., meta: _Optional[_Union[CheckResourcesResponse.ResultEntry.Meta, _Mapping]] = ..., outputs: _Optional[_Iterable[_Union[_engine_pb2.OutputEntry, _Mapping]]] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    results: _containers.RepeatedCompositeFieldContainer[CheckResourcesResponse.ResultEntry]
    def __init__(self, request_id: _Optional[str] = ..., results: _Optional[_Iterable[_Union[CheckResourcesResponse.ResultEntry, _Mapping]]] = ...) -> None: ...

class PlaygroundFailure(_message.Message):
    __slots__ = ["errors"]
    class Error(_message.Message):
        __slots__ = ["file", "error"]
        FILE_FIELD_NUMBER: _ClassVar[int]
        ERROR_FIELD_NUMBER: _ClassVar[int]
        file: str
        error: str
        def __init__(self, file: _Optional[str] = ..., error: _Optional[str] = ...) -> None: ...
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    errors: _containers.RepeatedCompositeFieldContainer[PlaygroundFailure.Error]
    def __init__(self, errors: _Optional[_Iterable[_Union[PlaygroundFailure.Error, _Mapping]]] = ...) -> None: ...

class PlaygroundValidateResponse(_message.Message):
    __slots__ = ["playground_id", "failure", "success"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FAILURE_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    failure: PlaygroundFailure
    success: _empty_pb2.Empty
    def __init__(self, playground_id: _Optional[str] = ..., failure: _Optional[_Union[PlaygroundFailure, _Mapping]] = ..., success: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ...) -> None: ...

class PlaygroundTestResponse(_message.Message):
    __slots__ = ["playground_id", "failure", "success"]
    class TestResults(_message.Message):
        __slots__ = ["results"]
        RESULTS_FIELD_NUMBER: _ClassVar[int]
        results: _policy_pb2.TestResults
        def __init__(self, results: _Optional[_Union[_policy_pb2.TestResults, _Mapping]] = ...) -> None: ...
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FAILURE_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    failure: PlaygroundFailure
    success: PlaygroundTestResponse.TestResults
    def __init__(self, playground_id: _Optional[str] = ..., failure: _Optional[_Union[PlaygroundFailure, _Mapping]] = ..., success: _Optional[_Union[PlaygroundTestResponse.TestResults, _Mapping]] = ...) -> None: ...

class PlaygroundEvaluateResponse(_message.Message):
    __slots__ = ["playground_id", "failure", "success"]
    class EvalResult(_message.Message):
        __slots__ = ["action", "effect", "policy", "effective_derived_roles", "validation_errors"]
        ACTION_FIELD_NUMBER: _ClassVar[int]
        EFFECT_FIELD_NUMBER: _ClassVar[int]
        POLICY_FIELD_NUMBER: _ClassVar[int]
        EFFECTIVE_DERIVED_ROLES_FIELD_NUMBER: _ClassVar[int]
        VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
        action: str
        effect: _effect_pb2.Effect
        policy: str
        effective_derived_roles: _containers.RepeatedScalarFieldContainer[str]
        validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
        def __init__(self, action: _Optional[str] = ..., effect: _Optional[_Union[_effect_pb2.Effect, str]] = ..., policy: _Optional[str] = ..., effective_derived_roles: _Optional[_Iterable[str]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ...) -> None: ...
    class EvalResultList(_message.Message):
        __slots__ = ["results", "effective_derived_roles", "validation_errors", "outputs"]
        RESULTS_FIELD_NUMBER: _ClassVar[int]
        EFFECTIVE_DERIVED_ROLES_FIELD_NUMBER: _ClassVar[int]
        VALIDATION_ERRORS_FIELD_NUMBER: _ClassVar[int]
        OUTPUTS_FIELD_NUMBER: _ClassVar[int]
        results: _containers.RepeatedCompositeFieldContainer[PlaygroundEvaluateResponse.EvalResult]
        effective_derived_roles: _containers.RepeatedScalarFieldContainer[str]
        validation_errors: _containers.RepeatedCompositeFieldContainer[_schema_pb2.ValidationError]
        outputs: _containers.RepeatedCompositeFieldContainer[_engine_pb2.OutputEntry]
        def __init__(self, results: _Optional[_Iterable[_Union[PlaygroundEvaluateResponse.EvalResult, _Mapping]]] = ..., effective_derived_roles: _Optional[_Iterable[str]] = ..., validation_errors: _Optional[_Iterable[_Union[_schema_pb2.ValidationError, _Mapping]]] = ..., outputs: _Optional[_Iterable[_Union[_engine_pb2.OutputEntry, _Mapping]]] = ...) -> None: ...
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FAILURE_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    failure: PlaygroundFailure
    success: PlaygroundEvaluateResponse.EvalResultList
    def __init__(self, playground_id: _Optional[str] = ..., failure: _Optional[_Union[PlaygroundFailure, _Mapping]] = ..., success: _Optional[_Union[PlaygroundEvaluateResponse.EvalResultList, _Mapping]] = ...) -> None: ...

class PlaygroundProxyResponse(_message.Message):
    __slots__ = ["playground_id", "failure", "check_resource_set", "check_resource_batch", "plan_resources", "check_resources"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FAILURE_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCE_SET_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCE_BATCH_FIELD_NUMBER: _ClassVar[int]
    PLAN_RESOURCES_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCES_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    failure: PlaygroundFailure
    check_resource_set: CheckResourceSetResponse
    check_resource_batch: CheckResourceBatchResponse
    plan_resources: PlanResourcesResponse
    check_resources: CheckResourcesResponse
    def __init__(self, playground_id: _Optional[str] = ..., failure: _Optional[_Union[PlaygroundFailure, _Mapping]] = ..., check_resource_set: _Optional[_Union[CheckResourceSetResponse, _Mapping]] = ..., check_resource_batch: _Optional[_Union[CheckResourceBatchResponse, _Mapping]] = ..., plan_resources: _Optional[_Union[PlanResourcesResponse, _Mapping]] = ..., check_resources: _Optional[_Union[CheckResourcesResponse, _Mapping]] = ...) -> None: ...

class AddOrUpdatePolicyResponse(_message.Message):
    __slots__ = ["success"]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    success: _empty_pb2.Empty
    def __init__(self, success: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ...) -> None: ...

class ListAuditLogEntriesResponse(_message.Message):
    __slots__ = ["access_log_entry", "decision_log_entry"]
    ACCESS_LOG_ENTRY_FIELD_NUMBER: _ClassVar[int]
    DECISION_LOG_ENTRY_FIELD_NUMBER: _ClassVar[int]
    access_log_entry: _audit_pb2.AccessLogEntry
    decision_log_entry: _audit_pb2.DecisionLogEntry
    def __init__(self, access_log_entry: _Optional[_Union[_audit_pb2.AccessLogEntry, _Mapping]] = ..., decision_log_entry: _Optional[_Union[_audit_pb2.DecisionLogEntry, _Mapping]] = ...) -> None: ...

class ServerInfoResponse(_message.Message):
    __slots__ = ["version", "commit", "build_date"]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    COMMIT_FIELD_NUMBER: _ClassVar[int]
    BUILD_DATE_FIELD_NUMBER: _ClassVar[int]
    version: str
    commit: str
    build_date: str
    def __init__(self, version: _Optional[str] = ..., commit: _Optional[str] = ..., build_date: _Optional[str] = ...) -> None: ...

class ListPoliciesResponse(_message.Message):
    __slots__ = ["policy_ids"]
    POLICY_IDS_FIELD_NUMBER: _ClassVar[int]
    policy_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, policy_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class GetPolicyResponse(_message.Message):
    __slots__ = ["policies"]
    POLICIES_FIELD_NUMBER: _ClassVar[int]
    policies: _containers.RepeatedCompositeFieldContainer[_policy_pb2.Policy]
    def __init__(self, policies: _Optional[_Iterable[_Union[_policy_pb2.Policy, _Mapping]]] = ...) -> None: ...

class DisablePolicyResponse(_message.Message):
    __slots__ = ["disabled_policies"]
    DISABLED_POLICIES_FIELD_NUMBER: _ClassVar[int]
    disabled_policies: int
    def __init__(self, disabled_policies: _Optional[int] = ...) -> None: ...

class EnablePolicyResponse(_message.Message):
    __slots__ = ["enabled_policies"]
    ENABLED_POLICIES_FIELD_NUMBER: _ClassVar[int]
    enabled_policies: int
    def __init__(self, enabled_policies: _Optional[int] = ...) -> None: ...

class AddOrUpdateSchemaResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListSchemasResponse(_message.Message):
    __slots__ = ["schema_ids"]
    SCHEMA_IDS_FIELD_NUMBER: _ClassVar[int]
    schema_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, schema_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class GetSchemaResponse(_message.Message):
    __slots__ = ["schemas"]
    SCHEMAS_FIELD_NUMBER: _ClassVar[int]
    schemas: _containers.RepeatedCompositeFieldContainer[_schema_pb2.Schema]
    def __init__(self, schemas: _Optional[_Iterable[_Union[_schema_pb2.Schema, _Mapping]]] = ...) -> None: ...

class DeleteSchemaResponse(_message.Message):
    __slots__ = ["deleted_schemas"]
    DELETED_SCHEMAS_FIELD_NUMBER: _ClassVar[int]
    deleted_schemas: int
    def __init__(self, deleted_schemas: _Optional[int] = ...) -> None: ...

class ReloadStoreResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



from cerbos.engine.v1 import engine_pb2 as _engine_pb2
from cerbos.policy.v1 import policy_pb2 as _policy_pb2
from cerbos.schema.v1 import schema_pb2 as _schema_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2
from validate import validate_pb2 as _validate_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PlanResourcesRequest(_message.Message):
    __slots__ = ["request_id", "action", "principal", "resource", "aux_data", "include_meta"]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    AUX_DATA_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_META_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    action: str
    principal: _engine_pb2.Principal
    resource: _engine_pb2.PlanResourcesInput.Resource
    aux_data: AuxData
    include_meta: bool
    def __init__(self, request_id: _Optional[str] = ..., action: _Optional[str] = ..., principal: _Optional[_Union[_engine_pb2.Principal, _Mapping]] = ..., resource: _Optional[_Union[_engine_pb2.PlanResourcesInput.Resource, _Mapping]] = ..., aux_data: _Optional[_Union[AuxData, _Mapping]] = ..., include_meta: bool = ...) -> None: ...

class CheckResourceSetRequest(_message.Message):
    __slots__ = ["request_id", "actions", "principal", "resource", "include_meta", "aux_data"]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    ACTIONS_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_META_FIELD_NUMBER: _ClassVar[int]
    AUX_DATA_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    actions: _containers.RepeatedScalarFieldContainer[str]
    principal: _engine_pb2.Principal
    resource: ResourceSet
    include_meta: bool
    aux_data: AuxData
    def __init__(self, request_id: _Optional[str] = ..., actions: _Optional[_Iterable[str]] = ..., principal: _Optional[_Union[_engine_pb2.Principal, _Mapping]] = ..., resource: _Optional[_Union[ResourceSet, _Mapping]] = ..., include_meta: bool = ..., aux_data: _Optional[_Union[AuxData, _Mapping]] = ...) -> None: ...

class ResourceSet(_message.Message):
    __slots__ = ["kind", "policy_version", "instances", "scope"]
    class InstancesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: AttributesMap
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[AttributesMap, _Mapping]] = ...) -> None: ...
    KIND_FIELD_NUMBER: _ClassVar[int]
    POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
    INSTANCES_FIELD_NUMBER: _ClassVar[int]
    SCOPE_FIELD_NUMBER: _ClassVar[int]
    kind: str
    policy_version: str
    instances: _containers.MessageMap[str, AttributesMap]
    scope: str
    def __init__(self, kind: _Optional[str] = ..., policy_version: _Optional[str] = ..., instances: _Optional[_Mapping[str, AttributesMap]] = ..., scope: _Optional[str] = ...) -> None: ...

class AttributesMap(_message.Message):
    __slots__ = ["attr"]
    class AttrEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...) -> None: ...
    ATTR_FIELD_NUMBER: _ClassVar[int]
    attr: _containers.MessageMap[str, _struct_pb2.Value]
    def __init__(self, attr: _Optional[_Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class CheckResourceBatchRequest(_message.Message):
    __slots__ = ["request_id", "principal", "resources", "aux_data"]
    class BatchEntry(_message.Message):
        __slots__ = ["actions", "resource"]
        ACTIONS_FIELD_NUMBER: _ClassVar[int]
        RESOURCE_FIELD_NUMBER: _ClassVar[int]
        actions: _containers.RepeatedScalarFieldContainer[str]
        resource: _engine_pb2.Resource
        def __init__(self, actions: _Optional[_Iterable[str]] = ..., resource: _Optional[_Union[_engine_pb2.Resource, _Mapping]] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    AUX_DATA_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    principal: _engine_pb2.Principal
    resources: _containers.RepeatedCompositeFieldContainer[CheckResourceBatchRequest.BatchEntry]
    aux_data: AuxData
    def __init__(self, request_id: _Optional[str] = ..., principal: _Optional[_Union[_engine_pb2.Principal, _Mapping]] = ..., resources: _Optional[_Iterable[_Union[CheckResourceBatchRequest.BatchEntry, _Mapping]]] = ..., aux_data: _Optional[_Union[AuxData, _Mapping]] = ...) -> None: ...

class CheckResourcesRequest(_message.Message):
    __slots__ = ["request_id", "include_meta", "principal", "resources", "aux_data"]
    class ResourceEntry(_message.Message):
        __slots__ = ["actions", "resource"]
        ACTIONS_FIELD_NUMBER: _ClassVar[int]
        RESOURCE_FIELD_NUMBER: _ClassVar[int]
        actions: _containers.RepeatedScalarFieldContainer[str]
        resource: _engine_pb2.Resource
        def __init__(self, actions: _Optional[_Iterable[str]] = ..., resource: _Optional[_Union[_engine_pb2.Resource, _Mapping]] = ...) -> None: ...
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_META_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    AUX_DATA_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    include_meta: bool
    principal: _engine_pb2.Principal
    resources: _containers.RepeatedCompositeFieldContainer[CheckResourcesRequest.ResourceEntry]
    aux_data: AuxData
    def __init__(self, request_id: _Optional[str] = ..., include_meta: bool = ..., principal: _Optional[_Union[_engine_pb2.Principal, _Mapping]] = ..., resources: _Optional[_Iterable[_Union[CheckResourcesRequest.ResourceEntry, _Mapping]]] = ..., aux_data: _Optional[_Union[AuxData, _Mapping]] = ...) -> None: ...

class AuxData(_message.Message):
    __slots__ = ["jwt"]
    class JWT(_message.Message):
        __slots__ = ["token", "key_set_id"]
        TOKEN_FIELD_NUMBER: _ClassVar[int]
        KEY_SET_ID_FIELD_NUMBER: _ClassVar[int]
        token: str
        key_set_id: str
        def __init__(self, token: _Optional[str] = ..., key_set_id: _Optional[str] = ...) -> None: ...
    JWT_FIELD_NUMBER: _ClassVar[int]
    jwt: AuxData.JWT
    def __init__(self, jwt: _Optional[_Union[AuxData.JWT, _Mapping]] = ...) -> None: ...

class File(_message.Message):
    __slots__ = ["file_name", "contents"]
    FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    CONTENTS_FIELD_NUMBER: _ClassVar[int]
    file_name: str
    contents: bytes
    def __init__(self, file_name: _Optional[str] = ..., contents: _Optional[bytes] = ...) -> None: ...

class PlaygroundValidateRequest(_message.Message):
    __slots__ = ["playground_id", "files"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    files: _containers.RepeatedCompositeFieldContainer[File]
    def __init__(self, playground_id: _Optional[str] = ..., files: _Optional[_Iterable[_Union[File, _Mapping]]] = ...) -> None: ...

class PlaygroundTestRequest(_message.Message):
    __slots__ = ["playground_id", "files"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    files: _containers.RepeatedCompositeFieldContainer[File]
    def __init__(self, playground_id: _Optional[str] = ..., files: _Optional[_Iterable[_Union[File, _Mapping]]] = ...) -> None: ...

class PlaygroundEvaluateRequest(_message.Message):
    __slots__ = ["playground_id", "files", "principal", "resource", "actions", "aux_data"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    ACTIONS_FIELD_NUMBER: _ClassVar[int]
    AUX_DATA_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    files: _containers.RepeatedCompositeFieldContainer[File]
    principal: _engine_pb2.Principal
    resource: _engine_pb2.Resource
    actions: _containers.RepeatedScalarFieldContainer[str]
    aux_data: AuxData
    def __init__(self, playground_id: _Optional[str] = ..., files: _Optional[_Iterable[_Union[File, _Mapping]]] = ..., principal: _Optional[_Union[_engine_pb2.Principal, _Mapping]] = ..., resource: _Optional[_Union[_engine_pb2.Resource, _Mapping]] = ..., actions: _Optional[_Iterable[str]] = ..., aux_data: _Optional[_Union[AuxData, _Mapping]] = ...) -> None: ...

class PlaygroundProxyRequest(_message.Message):
    __slots__ = ["playground_id", "files", "check_resource_set", "check_resource_batch", "plan_resources", "check_resources"]
    PLAYGROUND_ID_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCE_SET_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCE_BATCH_FIELD_NUMBER: _ClassVar[int]
    PLAN_RESOURCES_FIELD_NUMBER: _ClassVar[int]
    CHECK_RESOURCES_FIELD_NUMBER: _ClassVar[int]
    playground_id: str
    files: _containers.RepeatedCompositeFieldContainer[File]
    check_resource_set: CheckResourceSetRequest
    check_resource_batch: CheckResourceBatchRequest
    plan_resources: PlanResourcesRequest
    check_resources: CheckResourcesRequest
    def __init__(self, playground_id: _Optional[str] = ..., files: _Optional[_Iterable[_Union[File, _Mapping]]] = ..., check_resource_set: _Optional[_Union[CheckResourceSetRequest, _Mapping]] = ..., check_resource_batch: _Optional[_Union[CheckResourceBatchRequest, _Mapping]] = ..., plan_resources: _Optional[_Union[PlanResourcesRequest, _Mapping]] = ..., check_resources: _Optional[_Union[CheckResourcesRequest, _Mapping]] = ...) -> None: ...

class AddOrUpdatePolicyRequest(_message.Message):
    __slots__ = ["policies"]
    POLICIES_FIELD_NUMBER: _ClassVar[int]
    policies: _containers.RepeatedCompositeFieldContainer[_policy_pb2.Policy]
    def __init__(self, policies: _Optional[_Iterable[_Union[_policy_pb2.Policy, _Mapping]]] = ...) -> None: ...

class ListAuditLogEntriesRequest(_message.Message):
    __slots__ = ["kind", "tail", "between", "since", "lookup"]
    class Kind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        KIND_UNSPECIFIED: _ClassVar[ListAuditLogEntriesRequest.Kind]
        KIND_ACCESS: _ClassVar[ListAuditLogEntriesRequest.Kind]
        KIND_DECISION: _ClassVar[ListAuditLogEntriesRequest.Kind]
    KIND_UNSPECIFIED: ListAuditLogEntriesRequest.Kind
    KIND_ACCESS: ListAuditLogEntriesRequest.Kind
    KIND_DECISION: ListAuditLogEntriesRequest.Kind
    class TimeRange(_message.Message):
        __slots__ = ["start", "end"]
        START_FIELD_NUMBER: _ClassVar[int]
        END_FIELD_NUMBER: _ClassVar[int]
        start: _timestamp_pb2.Timestamp
        end: _timestamp_pb2.Timestamp
        def __init__(self, start: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    KIND_FIELD_NUMBER: _ClassVar[int]
    TAIL_FIELD_NUMBER: _ClassVar[int]
    BETWEEN_FIELD_NUMBER: _ClassVar[int]
    SINCE_FIELD_NUMBER: _ClassVar[int]
    LOOKUP_FIELD_NUMBER: _ClassVar[int]
    kind: ListAuditLogEntriesRequest.Kind
    tail: int
    between: ListAuditLogEntriesRequest.TimeRange
    since: _duration_pb2.Duration
    lookup: str
    def __init__(self, kind: _Optional[_Union[ListAuditLogEntriesRequest.Kind, str]] = ..., tail: _Optional[int] = ..., between: _Optional[_Union[ListAuditLogEntriesRequest.TimeRange, _Mapping]] = ..., since: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., lookup: _Optional[str] = ...) -> None: ...

class ServerInfoRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListPoliciesRequest(_message.Message):
    __slots__ = ["include_disabled", "name_regexp", "scope_regexp", "version_regexp"]
    INCLUDE_DISABLED_FIELD_NUMBER: _ClassVar[int]
    NAME_REGEXP_FIELD_NUMBER: _ClassVar[int]
    SCOPE_REGEXP_FIELD_NUMBER: _ClassVar[int]
    VERSION_REGEXP_FIELD_NUMBER: _ClassVar[int]
    include_disabled: bool
    name_regexp: str
    scope_regexp: str
    version_regexp: str
    def __init__(self, include_disabled: bool = ..., name_regexp: _Optional[str] = ..., scope_regexp: _Optional[str] = ..., version_regexp: _Optional[str] = ...) -> None: ...

class GetPolicyRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, id: _Optional[_Iterable[str]] = ...) -> None: ...

class DisablePolicyRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, id: _Optional[_Iterable[str]] = ...) -> None: ...

class EnablePolicyRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, id: _Optional[_Iterable[str]] = ...) -> None: ...

class AddOrUpdateSchemaRequest(_message.Message):
    __slots__ = ["schemas"]
    SCHEMAS_FIELD_NUMBER: _ClassVar[int]
    schemas: _containers.RepeatedCompositeFieldContainer[_schema_pb2.Schema]
    def __init__(self, schemas: _Optional[_Iterable[_Union[_schema_pb2.Schema, _Mapping]]] = ...) -> None: ...

class ListSchemasRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class GetSchemaRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, id: _Optional[_Iterable[str]] = ...) -> None: ...

class DeleteSchemaRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, id: _Optional[_Iterable[str]] = ...) -> None: ...

class ReloadStoreRequest(_message.Message):
    __slots__ = ["wait"]
    WAIT_FIELD_NUMBER: _ClassVar[int]
    wait: bool
    def __init__(self, wait: bool = ...) -> None: ...



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from protoc_gen_openapiv2.options import openapiv2_pb2 as _openapiv2_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
OPENAPIV2_SWAGGER_FIELD_NUMBER: _ClassVar[int]
openapiv2_swagger: _descriptor.FieldDescriptor
OPENAPIV2_OPERATION_FIELD_NUMBER: _ClassVar[int]
openapiv2_operation: _descriptor.FieldDescriptor
OPENAPIV2_SCHEMA_FIELD_NUMBER: _ClassVar[int]
openapiv2_schema: _descriptor.FieldDescriptor
OPENAPIV2_TAG_FIELD_NUMBER: _ClassVar[int]
openapiv2_tag: _descriptor.FieldDescriptor
OPENAPIV2_FIELD_FIELD_NUMBER: _ClassVar[int]
openapiv2_field: _descriptor.FieldDescriptor



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cerbos/engine/v1/engine.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cerbos.effect.v1 import effect_pb2 as cerbos_dot_effect_dot_v1_dot_effect__pb2
from cerbos.schema.v1 import schema_pb2 as cerbos_dot_schema_dot_v1_dot_schema__pb2
from google.api.expr.v1alpha1 import checked_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_checked__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x63\x65rbos/engine/v1/engine.proto\x12\x10\x63\x65rbos.engine.v1\x1a\x1d\x63\x65rbos/effect/v1/effect.proto\x1a\x1d\x63\x65rbos/schema/v1/schema.proto\x1a&google/api/expr/v1alpha1/checked.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a\x17validate/validate.proto\"\xf7\t\n\x12PlanResourcesInput\x12\x1d\n\nrequest_id\x18\x01 \x01(\tR\trequestId\x12\x16\n\x06\x61\x63tion\x18\x02 \x01(\tR\x06\x61\x63tion\x12\x39\n\tprincipal\x18\x03 \x01(\x0b\x32\x1b.cerbos.engine.v1.PrincipalR\tprincipal\x12I\n\x08resource\x18\x04 \x01(\x0b\x32-.cerbos.engine.v1.PlanResourcesInput.ResourceR\x08resource\x12\x34\n\x08\x61ux_data\x18\x05 \x01(\x0b\x32\x19.cerbos.engine.v1.AuxDataR\x07\x61uxData\x12!\n\x0cinclude_meta\x18\x06 \x01(\x08R\x0bincludeMeta\x1a\xca\x07\n\x08Resource\x12\xc8\x01\n\x04kind\x18\x01 \x01(\tB\xb3\x01\x92\x41\x62\x32\x0eResource kind.J\x0e\"album:object\"\x8a\x01?^[[:alpha:]][[:word:]\\@\\.\\-]*(\\:[[:alpha:]][[:word:]\\@\\.\\-]*)*$\xe2\x41\x01\x02\xfa\x42GrE\x10\x01\x32\x41^[[:alpha:]][[:word:]\\@\\.\\-/]*(\\:[[:alpha:]][[:word:]\\@\\.\\-/]*)*$R\x04kind\x12\xb8\x01\n\x04\x61ttr\x18\x02 \x03(\x0b\x32\x37.cerbos.engine.v1.PlanResourcesInput.Resource.AttrEntryBk\x92\x41`2^Key-value pairs of contextual data about the resource that are known at a time of the request.\xfa\x42\x05\x9a\x01\x02\x18\x01R\x04\x61ttr\x12\xdd\x01\n\x0epolicy_version\x18\x03 \x01(\tB\xb5\x01\x92\x41\x99\x01\x32|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\x8a\x01\r^[[:word:]]*$\xe2\x41\x01\x01\xfa\x42\x11r\x0f\x32\r^[[:word:]]*$R\rpolicyVersion\x12\x86\x02\n\x05scope\x18\x04 \x01(\tB\xef\x01\x92\x41\xb1\x01\x32}A dot-separated scope that describes the hierarchy this resource belongs to. This is used for determining policy inheritance.\x8a\x01/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\xe2\x41\x01\x01\xfa\x42\x33r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$R\x05scope\x1aO\n\tAttrEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\"\xa1\x04\n\x10PlanResourcesAst\x12\x46\n\nfilter_ast\x18\x01 \x01(\x0b\x32\'.cerbos.engine.v1.PlanResourcesAst.NodeR\tfilterAst\x1a\xbb\x01\n\x04Node\x12\x62\n\x11logical_operation\x18\x01 \x01(\x0b\x32\x33.cerbos.engine.v1.PlanResourcesAst.LogicalOperationH\x00R\x10logicalOperation\x12G\n\nexpression\x18\x02 \x01(\x0b\x32%.google.api.expr.v1alpha1.CheckedExprH\x00R\nexpressionB\x06\n\x04node\x1a\x86\x02\n\x10LogicalOperation\x12X\n\x08operator\x18\x01 \x01(\x0e\x32<.cerbos.engine.v1.PlanResourcesAst.LogicalOperation.OperatorR\x08operator\x12=\n\x05nodes\x18\x02 \x03(\x0b\x32\'.cerbos.engine.v1.PlanResourcesAst.NodeR\x05nodes\"Y\n\x08Operator\x12\x18\n\x14OPERATOR_UNSPECIFIED\x10\x00\x12\x10\n\x0cOPERATOR_AND\x10\x01\x12\x0f\n\x0bOPERATOR_OR\x10\x02\x12\x10\n\x0cOPERATOR_NOT\x10\x03\"\xa4\x06\n\x13PlanResourcesFilter\x12\xad\x01\n\x04kind\x18\x01 \x01(\x0e\x32*.cerbos.engine.v1.PlanResourcesFilter.KindBm\x92\x41j2hFilter kind. Defines whether the given action is always allowed, always denied or allowed conditionally.R\x04kind\x12\x9a\x01\n\tcondition\x18\x02 \x01(\x0b\x32\x38.cerbos.engine.v1.PlanResourcesFilter.Expression.OperandBB\x92\x41?2=Filter condition. Only populated if kind is KIND_CONDITIONAL.R\tcondition\x1a\xda\x02\n\nExpression\x12)\n\x08operator\x18\x01 \x01(\tB\r\x92\x41\n2\x08OperatorR\x08operator\x12T\n\x08operands\x18\x02 \x03(\x0b\x32\x38.cerbos.engine.v1.PlanResourcesFilter.Expression.OperandR\x08operands\x1a\xb3\x01\n\x07Operand\x12.\n\x05value\x18\x01 \x01(\x0b\x32\x16.google.protobuf.ValueH\x00R\x05value\x12R\n\nexpression\x18\x02 \x01(\x0b\x32\x30.cerbos.engine.v1.PlanResourcesFilter.ExpressionH\x00R\nexpression\x12\x1c\n\x08variable\x18\x03 \x01(\tH\x00R\x08variableB\x06\n\x04node:\x15\x92\x41\x12\n\x10\x32\x0e\x43\x45L expression\"c\n\x04Kind\x12\x14\n\x10KIND_UNSPECIFIED\x10\x00\x12\x17\n\x13KIND_ALWAYS_ALLOWED\x10\x01\x12\x16\n\x12KIND_ALWAYS_DENIED\x10\x02\x12\x14\n\x10KIND_CONDITIONAL\x10\x03\"\xcf\x02\n\x13PlanResourcesOutput\x12\x1d\n\nrequest_id\x18\x01 \x01(\tR\trequestId\x12\x16\n\x06\x61\x63tion\x18\x02 \x01(\tR\x06\x61\x63tion\x12\x12\n\x04kind\x18\x03 \x01(\tR\x04kind\x12%\n\x0epolicy_version\x18\x04 \x01(\tR\rpolicyVersion\x12\x14\n\x05scope\x18\x05 \x01(\tR\x05scope\x12=\n\x06\x66ilter\x18\x06 \x01(\x0b\x32%.cerbos.engine.v1.PlanResourcesFilterR\x06\x66ilter\x12!\n\x0c\x66ilter_debug\x18\x07 \x01(\tR\x0b\x66ilterDebug\x12N\n\x11validation_errors\x18\x08 \x03(\x0b\x32!.cerbos.schema.v1.ValidationErrorR\x10validationErrors\"\x9e\x02\n\nCheckInput\x12\x1d\n\nrequest_id\x18\x01 \x01(\tR\trequestId\x12\x44\n\x08resource\x18\x02 \x01(\x0b\x32\x1a.cerbos.engine.v1.ResourceB\x0c\xe2\x41\x01\x02\xfa\x42\x05\x8a\x01\x02\x10\x01R\x08resource\x12G\n\tprincipal\x18\x03 \x01(\x0b\x32\x1b.cerbos.engine.v1.PrincipalB\x0c\xe2\x41\x01\x02\xfa\x42\x05\x8a\x01\x02\x10\x01R\tprincipal\x12,\n\x07\x61\x63tions\x18\x04 \x03(\tB\x12\xe2\x41\x01\x02\xfa\x42\x0b\x92\x01\x08\x18\x01\"\x04r\x02\x10\x01R\x07\x61\x63tions\x12\x34\n\x08\x61ux_data\x18\x05 \x01(\x0b\x32\x19.cerbos.engine.v1.AuxDataR\x07\x61uxData\"\xac\x04\n\x0b\x43heckOutput\x12\x1d\n\nrequest_id\x18\x01 \x01(\tR\trequestId\x12\x1f\n\x0bresource_id\x18\x02 \x01(\tR\nresourceId\x12\x44\n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32*.cerbos.engine.v1.CheckOutput.ActionsEntryR\x07\x61\x63tions\x12\x36\n\x17\x65\x66\x66\x65\x63tive_derived_roles\x18\x04 \x03(\tR\x15\x65\x66\x66\x65\x63tiveDerivedRoles\x12N\n\x11validation_errors\x18\x05 \x03(\x0b\x32!.cerbos.schema.v1.ValidationErrorR\x10validationErrors\x12\x37\n\x07outputs\x18\x06 \x03(\x0b\x32\x1d.cerbos.engine.v1.OutputEntryR\x07outputs\x1an\n\x0c\x41\x63tionEffect\x12\x30\n\x06\x65\x66\x66\x65\x63t\x18\x01 \x01(\x0e\x32\x18.cerbos.effect.v1.EffectR\x06\x65\x66\x66\x65\x63t\x12\x16\n\x06policy\x18\x02 \x01(\tR\x06policy\x12\x14\n\x05scope\x18\x03 \x01(\tR\x05scope\x1a\x66\n\x0c\x41\x63tionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12@\n\x05value\x18\x02 \x01(\x0b\x32*.cerbos.engine.v1.CheckOutput.ActionEffectR\x05value:\x02\x38\x01\"\xeb\x01\n\x0bOutputEntry\x12\x65\n\x03src\x18\x01 \x01(\tBS\x92\x41P2)Rule that matched to produce this output.J#\"resource.expense.v1/acme#rule-001\"R\x03src\x12u\n\x03val\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueBK\x92\x41H27Dynamic output, determined by user defined rule output.J\r\"some_string\"R\x03val\"\xc4\x08\n\x08Resource\x12\xe2\x01\n\x04kind\x18\x01 \x01(\tB\xcd\x01\x92\x41|2)Name of the resource kind being accessed.J\r\"album:photo\"\x8a\x01?^[[:alpha:]][[:word:]\\@\\.\\-]*(\\:[[:alpha:]][[:word:]\\@\\.\\-]*)*$\xe2\x41\x01\x02\xfa\x42GrE\x10\x01\x32\x41^[[:alpha:]][[:word:]\\@\\.\\-/]*(\\:[[:alpha:]][[:word:]\\@\\.\\-/]*)*$R\x04kind\x12\xdd\x01\n\x0epolicy_version\x18\x02 \x01(\tB\xb5\x01\x92\x41\x99\x01\x32|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\x8a\x01\r^[[:word:]]*$\xe2\x41\x01\x01\xfa\x42\x11r\x0f\x32\r^[[:word:]]*$R\rpolicyVersion\x12\x44\n\x02id\x18\x03 \x01(\tB4\x92\x41&2\x1bID of the resource instanceJ\x07\"XX125\"\xe2\x41\x01\x02\xfa\x42\x04r\x02\x10\x01R\x02id\x12\xc5\x01\n\x04\x61ttr\x18\x04 \x03(\x0b\x32$.cerbos.engine.v1.Resource.AttrEntryB\x8a\x01\x92\x41\x7f\x32\x64Kay-value pairs of contextual data about this resource that should be used during policy evaluation.J\x17{\"owner\": \"bugs_bunny\"}\xfa\x42\x05\x9a\x01\x02\x18\x01R\x04\x61ttr\x12\x93\x02\n\x05scope\x18\x05 \x01(\tB\xfc\x01\x92\x41\xbe\x01\x32}A dot-separated scope that describes the hierarchy this resource belongs to. This is used for determining policy inheritance.J\x0b\"acme.corp\"\x8a\x01/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\xe2\x41\x01\x01\xfa\x42\x33r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$R\x05scope\x1aO\n\tAttrEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\"\xe7\x08\n\tPrincipal\x12\x41\n\x02id\x18\x01 \x01(\tB1\x92\x41#2\x13ID of the principalJ\x0c\"bugs_bunny\"\xe2\x41\x01\x02\xfa\x42\x04r\x02\x10\x01R\x02id\x12\xdd\x01\n\x0epolicy_version\x18\x02 \x01(\tB\xb5\x01\x92\x41\x99\x01\x32|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\x8a\x01\r^[[:word:]]*$\xe2\x41\x01\x01\xfa\x42\x11r\x0f\x32\r^[[:word:]]*$R\rpolicyVersion\x12\xab\x01\n\x05roles\x18\x03 \x03(\tB\x94\x01\x92\x41l2FRoles assigned to this principal from your identity management system.J\x08[\"user\"]\x8a\x01\x11^[[:word:]\\-\\.]+$\xa8\x01\x01\xb0\x01\x01\xe2\x41\x01\x02\xfa\x42\x1e\x92\x01\x1b\x08\x01\x18\x01\"\x15r\x13\x32\x11^[[:word:]\\-\\.]+$R\x05roles\x12\xc5\x01\n\x04\x61ttr\x18\x04 \x03(\x0b\x32%.cerbos.engine.v1.Principal.AttrEntryB\x89\x01\x92\x41~2eKey-value pairs of contextual data about this principal that should be used during policy evaluation.J\x15{\"beta_tester\": true}\xfa\x42\x05\x9a\x01\x02\x18\x01R\x04\x61ttr\x12\x94\x02\n\x05scope\x18\x05 \x01(\tB\xfd\x01\x92\x41\xbf\x01\x32~A dot-separated scope that describes the hierarchy this principal belongs to. This is used for determining policy inheritance.J\x0b\"acme.corp\"\x8a\x01/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\xe2\x41\x01\x01\xfa\x42\x33r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$R\x05scope\x1aO\n\tAttrEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01:Y\x92\x41V\nT2RA person or application attempting to perform the actions on the set of resources.\"\xb1\x01\n\x07\x41uxData\x12\x34\n\x03jwt\x18\x01 \x03(\x0b\x32\".cerbos.engine.v1.AuxData.JwtEntryR\x03jwt\x1aN\n\x08JwtEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01: \x92\x41\x1d\n\x1b\x32\x19Structured auxiliary data\"\x93\t\n\x05Trace\x12\x41\n\ncomponents\x18\x01 \x03(\x0b\x32!.cerbos.engine.v1.Trace.ComponentR\ncomponents\x12\x33\n\x05\x65vent\x18\x02 \x01(\x0b\x32\x1d.cerbos.engine.v1.Trace.EventR\x05\x65vent\x1a\xeb\x05\n\tComponent\x12:\n\x04kind\x18\x01 \x01(\x0e\x32&.cerbos.engine.v1.Trace.Component.KindR\x04kind\x12\x18\n\x06\x61\x63tion\x18\x02 \x01(\tH\x00R\x06\x61\x63tion\x12#\n\x0c\x64\x65rived_role\x18\x03 \x01(\tH\x00R\x0b\x64\x65rivedRole\x12\x14\n\x04\x65xpr\x18\x04 \x01(\tH\x00R\x04\x65xpr\x12\x16\n\x05index\x18\x05 \x01(\rH\x00R\x05index\x12\x18\n\x06policy\x18\x06 \x01(\tH\x00R\x06policy\x12\x1c\n\x08resource\x18\x07 \x01(\tH\x00R\x08resource\x12\x14\n\x04rule\x18\x08 \x01(\tH\x00R\x04rule\x12\x16\n\x05scope\x18\t \x01(\tH\x00R\x05scope\x12H\n\x08variable\x18\n \x01(\x0b\x32*.cerbos.engine.v1.Trace.Component.VariableH\x00R\x08variable\x12\x18\n\x06output\x18\x0b \x01(\tH\x00R\x06output\x1a\x32\n\x08Variable\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04\x65xpr\x18\x02 \x01(\tR\x04\x65xpr\"\xab\x02\n\x04Kind\x12\x14\n\x10KIND_UNSPECIFIED\x10\x00\x12\x0f\n\x0bKIND_ACTION\x10\x01\x12\x16\n\x12KIND_CONDITION_ALL\x10\x02\x12\x16\n\x12KIND_CONDITION_ANY\x10\x03\x12\x17\n\x13KIND_CONDITION_NONE\x10\x04\x12\x12\n\x0eKIND_CONDITION\x10\x05\x12\x15\n\x11KIND_DERIVED_ROLE\x10\x06\x12\r\n\tKIND_EXPR\x10\x07\x12\x0f\n\x0bKIND_POLICY\x10\x08\x12\x11\n\rKIND_RESOURCE\x10\t\x12\r\n\tKIND_RULE\x10\n\x12\x0e\n\nKIND_SCOPE\x10\x0b\x12\x11\n\rKIND_VARIABLE\x10\x0c\x12\x12\n\x0eKIND_VARIABLES\x10\r\x12\x0f\n\x0bKIND_OUTPUT\x10\x0e\x42\t\n\x07\x64\x65tails\x1a\xa3\x02\n\x05\x45vent\x12<\n\x06status\x18\x01 \x01(\x0e\x32$.cerbos.engine.v1.Trace.Event.StatusR\x06status\x12\x30\n\x06\x65\x66\x66\x65\x63t\x18\x02 \x01(\x0e\x32\x18.cerbos.effect.v1.EffectR\x06\x65\x66\x66\x65\x63t\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rror\x12\x18\n\x07message\x18\x04 \x01(\tR\x07message\x12.\n\x06result\x18\x05 \x01(\x0b\x32\x16.google.protobuf.ValueR\x06result\"J\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x14\n\x10STATUS_ACTIVATED\x10\x01\x12\x12\n\x0eSTATUS_SKIPPED\x10\x02\x42o\n\x18\x64\x65v.cerbos.api.v1.engineZ<github.com/cerbos/cerbos/api/genpb/cerbos/engine/v1;enginev1\xaa\x02\x14\x43\x65rbos.Api.V1.Engineb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cerbos.engine.v1.engine_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030dev.cerbos.api.v1.engineZ<github.com/cerbos/cerbos/api/genpb/cerbos/engine/v1;enginev1\252\002\024Cerbos.Api.V1.Engine'
  _PLANRESOURCESINPUT_RESOURCE_ATTRENTRY._options = None
  _PLANRESOURCESINPUT_RESOURCE_ATTRENTRY._serialized_options = b'8\001'
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['kind']._options = None
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['kind']._serialized_options = b'\222Ab2\016Resource kind.J\016\"album:object\"\212\001?^[[:alpha:]][[:word:]\\@\\.\\-]*(\\:[[:alpha:]][[:word:]\\@\\.\\-]*)*$\342A\001\002\372BGrE\020\0012A^[[:alpha:]][[:word:]\\@\\.\\-/]*(\\:[[:alpha:]][[:word:]\\@\\.\\-/]*)*$'
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['attr']._options = None
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['attr']._serialized_options = b'\222A`2^Key-value pairs of contextual data about the resource that are known at a time of the request.\372B\005\232\001\002\030\001'
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['policy_version']._options = None
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['policy_version']._serialized_options = b'\222A\231\0012|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\212\001\r^[[:word:]]*$\342A\001\001\372B\021r\0172\r^[[:word:]]*$'
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['scope']._options = None
  _PLANRESOURCESINPUT_RESOURCE.fields_by_name['scope']._serialized_options = b'\222A\261\0012}A dot-separated scope that describes the hierarchy this resource belongs to. This is used for determining policy inheritance.\212\001/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\342A\001\001\372B3r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$'
  _PLANRESOURCESFILTER_EXPRESSION.fields_by_name['operator']._options = None
  _PLANRESOURCESFILTER_EXPRESSION.fields_by_name['operator']._serialized_options = b'\222A\n2\010Operator'
  _PLANRESOURCESFILTER_EXPRESSION._options = None
  _PLANRESOURCESFILTER_EXPRESSION._serialized_options = b'\222A\022\n\0202\016CEL expression'
  _PLANRESOURCESFILTER.fields_by_name['kind']._options = None
  _PLANRESOURCESFILTER.fields_by_name['kind']._serialized_options = b'\222Aj2hFilter kind. Defines whether the given action is always allowed, always denied or allowed conditionally.'
  _PLANRESOURCESFILTER.fields_by_name['condition']._options = None
  _PLANRESOURCESFILTER.fields_by_name['condition']._serialized_options = b'\222A?2=Filter condition. Only populated if kind is KIND_CONDITIONAL.'
  _CHECKINPUT.fields_by_name['resource']._options = None
  _CHECKINPUT.fields_by_name['resource']._serialized_options = b'\342A\001\002\372B\005\212\001\002\020\001'
  _CHECKINPUT.fields_by_name['principal']._options = None
  _CHECKINPUT.fields_by_name['principal']._serialized_options = b'\342A\001\002\372B\005\212\001\002\020\001'
  _CHECKINPUT.fields_by_name['actions']._options = None
  _CHECKINPUT.fields_by_name['actions']._serialized_options = b'\342A\001\002\372B\013\222\001\010\030\001\"\004r\002\020\001'
  _CHECKOUTPUT_ACTIONSENTRY._options = None
  _CHECKOUTPUT_ACTIONSENTRY._serialized_options = b'8\001'
  _OUTPUTENTRY.fields_by_name['src']._options = None
  _OUTPUTENTRY.fields_by_name['src']._serialized_options = b'\222AP2)Rule that matched to produce this output.J#\"resource.expense.v1/acme#rule-001\"'
  _OUTPUTENTRY.fields_by_name['val']._options = None
  _OUTPUTENTRY.fields_by_name['val']._serialized_options = b'\222AH27Dynamic output, determined by user defined rule output.J\r\"some_string\"'
  _RESOURCE_ATTRENTRY._options = None
  _RESOURCE_ATTRENTRY._serialized_options = b'8\001'
  _RESOURCE.fields_by_name['kind']._options = None
  _RESOURCE.fields_by_name['kind']._serialized_options = b'\222A|2)Name of the resource kind being accessed.J\r\"album:photo\"\212\001?^[[:alpha:]][[:word:]\\@\\.\\-]*(\\:[[:alpha:]][[:word:]\\@\\.\\-]*)*$\342A\001\002\372BGrE\020\0012A^[[:alpha:]][[:word:]\\@\\.\\-/]*(\\:[[:alpha:]][[:word:]\\@\\.\\-/]*)*$'
  _RESOURCE.fields_by_name['policy_version']._options = None
  _RESOURCE.fields_by_name['policy_version']._serialized_options = b'\222A\231\0012|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\212\001\r^[[:word:]]*$\342A\001\001\372B\021r\0172\r^[[:word:]]*$'
  _RESOURCE.fields_by_name['id']._options = None
  _RESOURCE.fields_by_name['id']._serialized_options = b'\222A&2\033ID of the resource instanceJ\007\"XX125\"\342A\001\002\372B\004r\002\020\001'
  _RESOURCE.fields_by_name['attr']._options = None
  _RESOURCE.fields_by_name['attr']._serialized_options = b'\222A\1772dKay-value pairs of contextual data about this resource that should be used during policy evaluation.J\027{\"owner\": \"bugs_bunny\"}\372B\005\232\001\002\030\001'
  _RESOURCE.fields_by_name['scope']._options = None
  _RESOURCE.fields_by_name['scope']._serialized_options = b'\222A\276\0012}A dot-separated scope that describes the hierarchy this resource belongs to. This is used for determining policy inheritance.J\013\"acme.corp\"\212\001/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\342A\001\001\372B3r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$'
  _PRINCIPAL_ATTRENTRY._options = None
  _PRINCIPAL_ATTRENTRY._serialized_options = b'8\001'
  _PRINCIPAL.fields_by_name['id']._options = None
  _PRINCIPAL.fields_by_name['id']._serialized_options = b'\222A#2\023ID of the principalJ\014\"bugs_bunny\"\342A\001\002\372B\004r\002\020\001'
  _PRINCIPAL.fields_by_name['policy_version']._options = None
  _PRINCIPAL.fields_by_name['policy_version']._serialized_options = b'\222A\231\0012|The policy version to use to evaluate this request. If not specified, will default to the server-configured default version.J\t\"default\"\212\001\r^[[:word:]]*$\342A\001\001\372B\021r\0172\r^[[:word:]]*$'
  _PRINCIPAL.fields_by_name['roles']._options = None
  _PRINCIPAL.fields_by_name['roles']._serialized_options = b'\222Al2FRoles assigned to this principal from your identity management system.J\010[\"user\"]\212\001\021^[[:word:]\\-\\.]+$\250\001\001\260\001\001\342A\001\002\372B\036\222\001\033\010\001\030\001\"\025r\0232\021^[[:word:]\\-\\.]+$'
  _PRINCIPAL.fields_by_name['attr']._options = None
  _PRINCIPAL.fields_by_name['attr']._serialized_options = b'\222A~2eKey-value pairs of contextual data about this principal that should be used during policy evaluation.J\025{\"beta_tester\": true}\372B\005\232\001\002\030\001'
  _PRINCIPAL.fields_by_name['scope']._options = None
  _PRINCIPAL.fields_by_name['scope']._serialized_options = b'\222A\277\0012~A dot-separated scope that describes the hierarchy this principal belongs to. This is used for determining policy inheritance.J\013\"acme.corp\"\212\001/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$\342A\001\001\372B3r12/^([[:alnum:]][[:word:]\\-]*(\\.[[:word:]\\-]*)*)*$'
  _PRINCIPAL._options = None
  _PRINCIPAL._serialized_options = b'\222AV\nT2RA person or application attempting to perform the actions on the set of resources.'
  _AUXDATA_JWTENTRY._options = None
  _AUXDATA_JWTENTRY._serialized_options = b'8\001'
  _AUXDATA._options = None
  _AUXDATA._serialized_options = b'\222A\035\n\0332\031Structured auxiliary data'
  _globals['_PLANRESOURCESINPUT']._serialized_start=290
  _globals['_PLANRESOURCESINPUT']._serialized_end=1561
  _globals['_PLANRESOURCESINPUT_RESOURCE']._serialized_start=591
  _globals['_PLANRESOURCESINPUT_RESOURCE']._serialized_end=1561
  _globals['_PLANRESOURCESINPUT_RESOURCE_ATTRENTRY']._serialized_start=1482
  _globals['_PLANRESOURCESINPUT_RESOURCE_ATTRENTRY']._serialized_end=1561
  _globals['_PLANRESOURCESAST']._serialized_start=1564
  _globals['_PLANRESOURCESAST']._serialized_end=2109
  _globals['_PLANRESOURCESAST_NODE']._serialized_start=1657
  _globals['_PLANRESOURCESAST_NODE']._serialized_end=1844
  _globals['_PLANRESOURCESAST_LOGICALOPERATION']._serialized_start=1847
  _globals['_PLANRESOURCESAST_LOGICALOPERATION']._serialized_end=2109
  _globals['_PLANRESOURCESAST_LOGICALOPERATION_OPERATOR']._serialized_start=2020
  _globals['_PLANRESOURCESAST_LOGICALOPERATION_OPERATOR']._serialized_end=2109
  _globals['_PLANRESOURCESFILTER']._serialized_start=2112
  _globals['_PLANRESOURCESFILTER']._serialized_end=2916
  _globals['_PLANRESOURCESFILTER_EXPRESSION']._serialized_start=2469
  _globals['_PLANRESOURCESFILTER_EXPRESSION']._serialized_end=2815
  _globals['_PLANRESOURCESFILTER_EXPRESSION_OPERAND']._serialized_start=2613
  _globals['_PLANRESOURCESFILTER_EXPRESSION_OPERAND']._serialized_end=2792
  _globals['_PLANRESOURCESFILTER_KIND']._serialized_start=2817
  _globals['_PLANRESOURCESFILTER_KIND']._serialized_end=2916
  _globals['_PLANRESOURCESOUTPUT']._serialized_start=2919
  _globals['_PLANRESOURCESOUTPUT']._serialized_end=3254
  _globals['_CHECKINPUT']._serialized_start=3257
  _globals['_CHECKINPUT']._serialized_end=3543
  _globals['_CHECKOUTPUT']._serialized_start=3546
  _globals['_CHECKOUTPUT']._serialized_end=4102
  _globals['_CHECKOUTPUT_ACTIONEFFECT']._serialized_start=3888
  _globals['_CHECKOUTPUT_ACTIONEFFECT']._serialized_end=3998
  _globals['_CHECKOUTPUT_ACTIONSENTRY']._serialized_start=4000
  _globals['_CHECKOUTPUT_ACTIONSENTRY']._serialized_end=4102
  _globals['_OUTPUTENTRY']._serialized_start=4105
  _globals['_OUTPUTENTRY']._serialized_end=4340
  _globals['_RESOURCE']._serialized_start=4343
  _globals['_RESOURCE']._serialized_end=5435
  _globals['_RESOURCE_ATTRENTRY']._serialized_start=1482
  _globals['_RESOURCE_ATTRENTRY']._serialized_end=1561
  _globals['_PRINCIPAL']._serialized_start=5438
  _globals['_PRINCIPAL']._serialized_end=6565
  _globals['_PRINCIPAL_ATTRENTRY']._serialized_start=1482
  _globals['_PRINCIPAL_ATTRENTRY']._serialized_end=1561
  _globals['_AUXDATA']._serialized_start=6568
  _globals['_AUXDATA']._serialized_end=6745
  _globals['_AUXDATA_JWTENTRY']._serialized_start=6633
  _globals['_AUXDATA_JWTENTRY']._serialized_end=6711
  _globals['_TRACE']._serialized_start=6748
  _globals['_TRACE']._serialized_end=7919
  _globals['_TRACE_COMPONENT']._serialized_start=6878
  _globals['_TRACE_COMPONENT']._serialized_end=7625
  _globals['_TRACE_COMPONENT_VARIABLE']._serialized_start=7262
  _globals['_TRACE_COMPONENT_VARIABLE']._serialized_end=7312
  _globals['_TRACE_COMPONENT_KIND']._serialized_start=7315
  _globals['_TRACE_COMPONENT_KIND']._serialized_end=7614
  _globals['_TRACE_EVENT']._serialized_start=7628
  _globals['_TRACE_EVENT']._serialized_end=7919
  _globals['_TRACE_EVENT_STATUS']._serialized_start=7845
  _globals['_TRACE_EVENT_STATUS']._serialized_end=7919
# @@protoc_insertion_point(module_scope)
